# -*- coding: UTF-8 -*-
# Code generated by lark suite oapi sdk gen

from typing import *

from ....api import Request, Response, set_timeout, set_tenant_key, set_user_access_token, set_path_params, \
    set_query_params, set_response_stream, set_is_response_stream, FormData, FormDataFile
from ....config import Config
from ....consts import ACCESS_TOKEN_TYPE_TENANT, ACCESS_TOKEN_TYPE_USER, ACCESS_TOKEN_TYPE_APP
from .model import *


class Service(object):
    def __init__(self, conf):
        # type: (Config) -> None
        self.conf = conf
        self.department_units = DepartmentUnitService(self)
        self.users = UserService(self)
        self.user_groups = UserGroupService(self)
        self.user_group_members = UserGroupMemberService(self)
        self.departments = DepartmentService(self)
        self.scopes = ScopeService(self)
        self.custom_attr_events = CustomAttrEventService(self)
        self.employee_type_enums = EmployeeTypeEnumService(self)
        



class DepartmentUnitService(object):
    def __init__(self, service):
        # type: (Service) -> None
        self.service = service

    def delete(self, tenant_key=None, timeout=None):
        # type: (str, int) -> DepartmentUnitDeleteReqCall

        request_opts = []   # type: List[Callable[[Any], Any]]

        if timeout is not None:
            request_opts += [set_timeout(timeout)]

        if tenant_key is not None:
            request_opts += [set_tenant_key(tenant_key)]

        return DepartmentUnitDeleteReqCall(self, request_opts=request_opts)

    def patch(self, body, tenant_key=None, timeout=None):
        # type: (DepartmentUnitPatchReqBody, str, int) -> DepartmentUnitPatchReqCall

        request_opts = []   # type: List[Callable[[Any], Any]]

        if timeout is not None:
            request_opts += [set_timeout(timeout)]

        if tenant_key is not None:
            request_opts += [set_tenant_key(tenant_key)]

        return DepartmentUnitPatchReqCall(self, body, request_opts=request_opts)

    def create(self, body, tenant_key=None, timeout=None):
        # type: (DepartmentUnit, str, int) -> DepartmentUnitCreateReqCall

        request_opts = []   # type: List[Callable[[Any], Any]]

        if timeout is not None:
            request_opts += [set_timeout(timeout)]

        if tenant_key is not None:
            request_opts += [set_tenant_key(tenant_key)]

        return DepartmentUnitCreateReqCall(self, body, request_opts=request_opts)


class UserService(object):
    def __init__(self, service):
        # type: (Service) -> None
        self.service = service

    def delete(self, body, tenant_key=None, timeout=None):
        # type: (UserDeleteReqBody, str, int) -> UserDeleteReqCall

        request_opts = []   # type: List[Callable[[Any], Any]]

        if timeout is not None:
            request_opts += [set_timeout(timeout)]

        if tenant_key is not None:
            request_opts += [set_tenant_key(tenant_key)]

        return UserDeleteReqCall(self, body, request_opts=request_opts)

    def update(self, body, tenant_key=None, timeout=None):
        # type: (User, str, int) -> UserUpdateReqCall

        request_opts = []   # type: List[Callable[[Any], Any]]

        if timeout is not None:
            request_opts += [set_timeout(timeout)]

        if tenant_key is not None:
            request_opts += [set_tenant_key(tenant_key)]

        return UserUpdateReqCall(self, body, request_opts=request_opts)

    def create(self, body, tenant_key=None, timeout=None):
        # type: (User, str, int) -> UserCreateReqCall

        request_opts = []   # type: List[Callable[[Any], Any]]

        if timeout is not None:
            request_opts += [set_timeout(timeout)]

        if tenant_key is not None:
            request_opts += [set_tenant_key(tenant_key)]

        return UserCreateReqCall(self, body, request_opts=request_opts)

    def patch(self, body, tenant_key=None, user_access_token=None, timeout=None):
        # type: (User, str, str, int) -> UserPatchReqCall

        request_opts = []   # type: List[Callable[[Any], Any]]

        if timeout is not None:
            request_opts += [set_timeout(timeout)]

        if tenant_key is not None:
            request_opts += [set_tenant_key(tenant_key)]

        if user_access_token is not None:
            request_opts += [set_user_access_token(user_access_token)]

        return UserPatchReqCall(self, body, request_opts=request_opts)

    def update_user_id(self, body, tenant_key=None, timeout=None):
        # type: (UserUpdateUserIdReqBody, str, int) -> UserUpdateUserIdReqCall

        request_opts = []   # type: List[Callable[[Any], Any]]

        if timeout is not None:
            request_opts += [set_timeout(timeout)]

        if tenant_key is not None:
            request_opts += [set_tenant_key(tenant_key)]

        return UserUpdateUserIdReqCall(self, body, request_opts=request_opts)

    def get(self, tenant_key=None, user_access_token=None, timeout=None):
        # type: (str, str, int) -> UserGetReqCall

        request_opts = []   # type: List[Callable[[Any], Any]]

        if timeout is not None:
            request_opts += [set_timeout(timeout)]

        if tenant_key is not None:
            request_opts += [set_tenant_key(tenant_key)]

        if user_access_token is not None:
            request_opts += [set_user_access_token(user_access_token)]

        return UserGetReqCall(self, request_opts=request_opts)

    def list(self, tenant_key=None, user_access_token=None, timeout=None):
        # type: (str, str, int) -> UserListReqCall

        request_opts = []   # type: List[Callable[[Any], Any]]

        if timeout is not None:
            request_opts += [set_timeout(timeout)]

        if tenant_key is not None:
            request_opts += [set_tenant_key(tenant_key)]

        if user_access_token is not None:
            request_opts += [set_user_access_token(user_access_token)]

        return UserListReqCall(self, request_opts=request_opts)


class UserGroupService(object):
    def __init__(self, service):
        # type: (Service) -> None
        self.service = service

    def update_user_group_id(self, body, tenant_key=None, timeout=None):
        # type: (UserGroupUpdateUserGroupIdReqBody, str, int) -> UserGroupUpdateUserGroupIdReqCall

        request_opts = []   # type: List[Callable[[Any], Any]]

        if timeout is not None:
            request_opts += [set_timeout(timeout)]

        if tenant_key is not None:
            request_opts += [set_tenant_key(tenant_key)]

        return UserGroupUpdateUserGroupIdReqCall(self, body, request_opts=request_opts)

    def delete(self, tenant_key=None, timeout=None):
        # type: (str, int) -> UserGroupDeleteReqCall

        request_opts = []   # type: List[Callable[[Any], Any]]

        if timeout is not None:
            request_opts += [set_timeout(timeout)]

        if tenant_key is not None:
            request_opts += [set_tenant_key(tenant_key)]

        return UserGroupDeleteReqCall(self, request_opts=request_opts)

    def patch(self, body, tenant_key=None, timeout=None):
        # type: (UserGroup, str, int) -> UserGroupPatchReqCall

        request_opts = []   # type: List[Callable[[Any], Any]]

        if timeout is not None:
            request_opts += [set_timeout(timeout)]

        if tenant_key is not None:
            request_opts += [set_tenant_key(tenant_key)]

        return UserGroupPatchReqCall(self, body, request_opts=request_opts)

    def create(self, body, tenant_key=None, timeout=None):
        # type: (UserGroup, str, int) -> UserGroupCreateReqCall

        request_opts = []   # type: List[Callable[[Any], Any]]

        if timeout is not None:
            request_opts += [set_timeout(timeout)]

        if tenant_key is not None:
            request_opts += [set_tenant_key(tenant_key)]

        return UserGroupCreateReqCall(self, body, request_opts=request_opts)

    def list(self, tenant_key=None, timeout=None):
        # type: (str, int) -> UserGroupListReqCall

        request_opts = []   # type: List[Callable[[Any], Any]]

        if timeout is not None:
            request_opts += [set_timeout(timeout)]

        if tenant_key is not None:
            request_opts += [set_tenant_key(tenant_key)]

        return UserGroupListReqCall(self, request_opts=request_opts)

    def get(self, tenant_key=None, timeout=None):
        # type: (str, int) -> UserGroupGetReqCall

        request_opts = []   # type: List[Callable[[Any], Any]]

        if timeout is not None:
            request_opts += [set_timeout(timeout)]

        if tenant_key is not None:
            request_opts += [set_tenant_key(tenant_key)]

        return UserGroupGetReqCall(self, request_opts=request_opts)


class UserGroupMemberService(object):
    def __init__(self, service):
        # type: (Service) -> None
        self.service = service

    def create(self, body, tenant_key=None, timeout=None):
        # type: (UserGroupMemberCreateReqBody, str, int) -> UserGroupMemberCreateReqCall

        request_opts = []   # type: List[Callable[[Any], Any]]

        if timeout is not None:
            request_opts += [set_timeout(timeout)]

        if tenant_key is not None:
            request_opts += [set_tenant_key(tenant_key)]

        return UserGroupMemberCreateReqCall(self, body, request_opts=request_opts)

    def list(self, tenant_key=None, timeout=None):
        # type: (str, int) -> UserGroupMemberListReqCall

        request_opts = []   # type: List[Callable[[Any], Any]]

        if timeout is not None:
            request_opts += [set_timeout(timeout)]

        if tenant_key is not None:
            request_opts += [set_tenant_key(tenant_key)]

        return UserGroupMemberListReqCall(self, request_opts=request_opts)

    def delete(self, tenant_key=None, timeout=None):
        # type: (str, int) -> UserGroupMemberDeleteReqCall

        request_opts = []   # type: List[Callable[[Any], Any]]

        if timeout is not None:
            request_opts += [set_timeout(timeout)]

        if tenant_key is not None:
            request_opts += [set_tenant_key(tenant_key)]

        return UserGroupMemberDeleteReqCall(self, request_opts=request_opts)


class DepartmentService(object):
    def __init__(self, service):
        # type: (Service) -> None
        self.service = service

    def get(self, tenant_key=None, user_access_token=None, timeout=None):
        # type: (str, str, int) -> DepartmentGetReqCall

        request_opts = []   # type: List[Callable[[Any], Any]]

        if timeout is not None:
            request_opts += [set_timeout(timeout)]

        if tenant_key is not None:
            request_opts += [set_tenant_key(tenant_key)]

        if user_access_token is not None:
            request_opts += [set_user_access_token(user_access_token)]

        return DepartmentGetReqCall(self, request_opts=request_opts)

    def list(self, tenant_key=None, user_access_token=None, timeout=None):
        # type: (str, str, int) -> DepartmentListReqCall

        request_opts = []   # type: List[Callable[[Any], Any]]

        if timeout is not None:
            request_opts += [set_timeout(timeout)]

        if tenant_key is not None:
            request_opts += [set_tenant_key(tenant_key)]

        if user_access_token is not None:
            request_opts += [set_user_access_token(user_access_token)]

        return DepartmentListReqCall(self, request_opts=request_opts)

    def patch(self, body, tenant_key=None, timeout=None):
        # type: (Department, str, int) -> DepartmentPatchReqCall

        request_opts = []   # type: List[Callable[[Any], Any]]

        if timeout is not None:
            request_opts += [set_timeout(timeout)]

        if tenant_key is not None:
            request_opts += [set_tenant_key(tenant_key)]

        return DepartmentPatchReqCall(self, body, request_opts=request_opts)

    def create(self, body, tenant_key=None, timeout=None):
        # type: (Department, str, int) -> DepartmentCreateReqCall

        request_opts = []   # type: List[Callable[[Any], Any]]

        if timeout is not None:
            request_opts += [set_timeout(timeout)]

        if tenant_key is not None:
            request_opts += [set_tenant_key(tenant_key)]

        return DepartmentCreateReqCall(self, body, request_opts=request_opts)

    def delete(self, tenant_key=None, timeout=None):
        # type: (str, int) -> DepartmentDeleteReqCall

        request_opts = []   # type: List[Callable[[Any], Any]]

        if timeout is not None:
            request_opts += [set_timeout(timeout)]

        if tenant_key is not None:
            request_opts += [set_tenant_key(tenant_key)]

        return DepartmentDeleteReqCall(self, request_opts=request_opts)

    def update(self, body, tenant_key=None, timeout=None):
        # type: (Department, str, int) -> DepartmentUpdateReqCall

        request_opts = []   # type: List[Callable[[Any], Any]]

        if timeout is not None:
            request_opts += [set_timeout(timeout)]

        if tenant_key is not None:
            request_opts += [set_tenant_key(tenant_key)]

        return DepartmentUpdateReqCall(self, body, request_opts=request_opts)

    def parent(self, tenant_key=None, user_access_token=None, timeout=None):
        # type: (str, str, int) -> DepartmentParentReqCall

        request_opts = []   # type: List[Callable[[Any], Any]]

        if timeout is not None:
            request_opts += [set_timeout(timeout)]

        if tenant_key is not None:
            request_opts += [set_tenant_key(tenant_key)]

        if user_access_token is not None:
            request_opts += [set_user_access_token(user_access_token)]

        return DepartmentParentReqCall(self, request_opts=request_opts)

    def update_department_id(self, body, tenant_key=None, timeout=None):
        # type: (DepartmentUpdateDepartmentIdReqBody, str, int) -> DepartmentUpdateDepartmentIdReqCall

        request_opts = []   # type: List[Callable[[Any], Any]]

        if timeout is not None:
            request_opts += [set_timeout(timeout)]

        if tenant_key is not None:
            request_opts += [set_tenant_key(tenant_key)]

        return DepartmentUpdateDepartmentIdReqCall(self, body, request_opts=request_opts)

    def unbind_department_chat(self, body, tenant_key=None, timeout=None):
        # type: (DepartmentUnbindDepartmentChatReqBody, str, int) -> DepartmentUnbindDepartmentChatReqCall

        request_opts = []   # type: List[Callable[[Any], Any]]

        if timeout is not None:
            request_opts += [set_timeout(timeout)]

        if tenant_key is not None:
            request_opts += [set_tenant_key(tenant_key)]

        return DepartmentUnbindDepartmentChatReqCall(self, body, request_opts=request_opts)

    def search(self, body, user_access_token=None, timeout=None):
        # type: (DepartmentSearchReqBody, str, int) -> DepartmentSearchReqCall

        request_opts = []   # type: List[Callable[[Any], Any]]

        if timeout is not None:
            request_opts += [set_timeout(timeout)]

        if user_access_token is not None:
            request_opts += [set_user_access_token(user_access_token)]

        return DepartmentSearchReqCall(self, body, request_opts=request_opts)


class ScopeService(object):
    def __init__(self, service):
        # type: (Service) -> None
        self.service = service


class CustomAttrEventService(object):
    def __init__(self, service):
        # type: (Service) -> None
        self.service = service


class EmployeeTypeEnumService(object):
    def __init__(self, service):
        # type: (Service) -> None
        self.service = service

    def list(self, tenant_key=None, timeout=None):
        # type: (str, int) -> EmployeeTypeEnumListReqCall

        request_opts = []   # type: List[Callable[[Any], Any]]

        if timeout is not None:
            request_opts += [set_timeout(timeout)]

        if tenant_key is not None:
            request_opts += [set_tenant_key(tenant_key)]

        return EmployeeTypeEnumListReqCall(self, request_opts=request_opts)



class DepartmentGetReqCall(object):
    def __init__(self, service, request_opts=None):
        # type: (DepartmentService, List[Any]) -> None

        self.service = service
        
        self.path_params = {}   # type: Dict[str, Any]
        self.query_params = {}  # type: Dict[str, Any]

        if request_opts:
            self.request_opts = request_opts
        else:
            self.request_opts = []  # type: List[Any]

    def set_department_id(self, department_id):
        # type: (str) -> DepartmentGetReqCall
        self.path_params['department_id'] = department_id
        return self

    def set_user_id_type(self, user_id_type):
        # type: (str) -> DepartmentGetReqCall
        self.query_params['user_id_type'] = user_id_type
        return self

    def set_department_id_type(self, department_id_type):
        # type: (str) -> DepartmentGetReqCall
        self.query_params['department_id_type'] = department_id_type
        return self

    def do(self):
        # type: () -> Response[DepartmentGetResult]
        root_service = self.service.service

        conf = root_service.conf
        self.request_opts += [set_path_params(self.path_params)]
        self.request_opts += [set_query_params(self.query_params)]
        req = Request('contact/v3/departments/:department_id', 'GET', [ACCESS_TOKEN_TYPE_TENANT, ACCESS_TOKEN_TYPE_USER],
                      None, output_class=DepartmentGetResult, request_opts=self.request_opts)
        resp = req.do(conf)
        return resp


class UserDeleteReqCall(object):
    def __init__(self, service, body, request_opts=None):
        # type: (UserService, UserDeleteReqBody, List[Any]) -> None

        self.service = service
        self.body = body
        self.path_params = {}   # type: Dict[str, Any]
        self.query_params = {}  # type: Dict[str, Any]

        if request_opts:
            self.request_opts = request_opts
        else:
            self.request_opts = []  # type: List[Any]

    def set_user_id(self, user_id):
        # type: (str) -> UserDeleteReqCall
        self.path_params['user_id'] = user_id
        return self

    def set_user_id_type(self, user_id_type):
        # type: (str) -> UserDeleteReqCall
        self.query_params['user_id_type'] = user_id_type
        return self

    def do(self):
        # type: () -> Response[None]
        root_service = self.service.service

        conf = root_service.conf
        self.request_opts += [set_path_params(self.path_params)]
        self.request_opts += [set_query_params(self.query_params)]
        req = Request('contact/v3/users/:user_id', 'DELETE', [ACCESS_TOKEN_TYPE_TENANT],
                      self.body, request_opts=self.request_opts)
        resp = req.do(conf)
        return resp


class DepartmentListReqCall(object):
    def __init__(self, service, request_opts=None):
        # type: (DepartmentService, List[Any]) -> None

        self.service = service
        
        self.query_params = {}  # type: Dict[str, Any]

        if request_opts:
            self.request_opts = request_opts
        else:
            self.request_opts = []  # type: List[Any]

    def set_user_id_type(self, user_id_type):
        # type: (str) -> DepartmentListReqCall
        self.query_params['user_id_type'] = user_id_type
        return self

    def set_department_id_type(self, department_id_type):
        # type: (str) -> DepartmentListReqCall
        self.query_params['department_id_type'] = department_id_type
        return self

    def set_parent_department_id(self, parent_department_id):
        # type: (str) -> DepartmentListReqCall
        self.query_params['parent_department_id'] = parent_department_id
        return self

    def set_fetch_child(self, fetch_child):
        # type: (bool) -> DepartmentListReqCall
        self.query_params['fetch_child'] = fetch_child
        return self

    def set_page_token(self, page_token):
        # type: (str) -> DepartmentListReqCall
        self.query_params['page_token'] = page_token
        return self

    def set_page_size(self, page_size):
        # type: (int) -> DepartmentListReqCall
        self.query_params['page_size'] = page_size
        return self

    def do(self):
        # type: () -> Response[DepartmentListResult]
        root_service = self.service.service

        conf = root_service.conf
        self.request_opts += [set_query_params(self.query_params)]
        req = Request('contact/v3/departments', 'GET', [ACCESS_TOKEN_TYPE_TENANT, ACCESS_TOKEN_TYPE_USER],
                      None, output_class=DepartmentListResult, request_opts=self.request_opts)
        resp = req.do(conf)
        return resp


class UserGroupUpdateUserGroupIdReqCall(object):
    def __init__(self, service, body, request_opts=None):
        # type: (UserGroupService, UserGroupUpdateUserGroupIdReqBody, List[Any]) -> None

        self.service = service
        self.body = body
        self.path_params = {}   # type: Dict[str, Any]

        if request_opts:
            self.request_opts = request_opts
        else:
            self.request_opts = []  # type: List[Any]

    def set_user_group_id(self, user_group_id):
        # type: (str) -> UserGroupUpdateUserGroupIdReqCall
        self.path_params['user_group_id'] = user_group_id
        return self

    def do(self):
        # type: () -> Response[None]
        root_service = self.service.service

        conf = root_service.conf
        self.request_opts += [set_path_params(self.path_params)]
        req = Request('contact/v3/user_groups/:user_group_id/update_user_group_id', 'PATCH', [ACCESS_TOKEN_TYPE_TENANT],
                      self.body, request_opts=self.request_opts)
        resp = req.do(conf)
        return resp


class DepartmentUnitDeleteReqCall(object):
    def __init__(self, service, request_opts=None):
        # type: (DepartmentUnitService, List[Any]) -> None

        self.service = service
        
        self.path_params = {}   # type: Dict[str, Any]

        if request_opts:
            self.request_opts = request_opts
        else:
            self.request_opts = []  # type: List[Any]

    def set_unit_id(self, unit_id):
        # type: (str) -> DepartmentUnitDeleteReqCall
        self.path_params['unit_id'] = unit_id
        return self

    def do(self):
        # type: () -> Response[None]
        root_service = self.service.service

        conf = root_service.conf
        self.request_opts += [set_path_params(self.path_params)]
        req = Request('contact/v3/department_units/:unit_id', 'DELETE', [ACCESS_TOKEN_TYPE_TENANT],
                      None, request_opts=self.request_opts)
        resp = req.do(conf)
        return resp


class DepartmentPatchReqCall(object):
    def __init__(self, service, body, request_opts=None):
        # type: (DepartmentService, Department, List[Any]) -> None

        self.service = service
        self.body = body
        self.path_params = {}   # type: Dict[str, Any]
        self.query_params = {}  # type: Dict[str, Any]

        if request_opts:
            self.request_opts = request_opts
        else:
            self.request_opts = []  # type: List[Any]

    def set_department_id(self, department_id):
        # type: (str) -> DepartmentPatchReqCall
        self.path_params['department_id'] = department_id
        return self

    def set_user_id_type(self, user_id_type):
        # type: (str) -> DepartmentPatchReqCall
        self.query_params['user_id_type'] = user_id_type
        return self

    def set_department_id_type(self, department_id_type):
        # type: (str) -> DepartmentPatchReqCall
        self.query_params['department_id_type'] = department_id_type
        return self

    def do(self):
        # type: () -> Response[DepartmentPatchResult]
        root_service = self.service.service

        conf = root_service.conf
        self.request_opts += [set_path_params(self.path_params)]
        self.request_opts += [set_query_params(self.query_params)]
        req = Request('contact/v3/departments/:department_id', 'PATCH', [ACCESS_TOKEN_TYPE_TENANT],
                      self.body, output_class=DepartmentPatchResult, request_opts=self.request_opts)
        resp = req.do(conf)
        return resp


class UserUpdateReqCall(object):
    def __init__(self, service, body, request_opts=None):
        # type: (UserService, User, List[Any]) -> None

        self.service = service
        self.body = body
        self.path_params = {}   # type: Dict[str, Any]
        self.query_params = {}  # type: Dict[str, Any]

        if request_opts:
            self.request_opts = request_opts
        else:
            self.request_opts = []  # type: List[Any]

    def set_user_id(self, user_id):
        # type: (str) -> UserUpdateReqCall
        self.path_params['user_id'] = user_id
        return self

    def set_user_id_type(self, user_id_type):
        # type: (str) -> UserUpdateReqCall
        self.query_params['user_id_type'] = user_id_type
        return self

    def set_department_id_type(self, department_id_type):
        # type: (str) -> UserUpdateReqCall
        self.query_params['department_id_type'] = department_id_type
        return self

    def do(self):
        # type: () -> Response[UserUpdateResult]
        root_service = self.service.service

        conf = root_service.conf
        self.request_opts += [set_path_params(self.path_params)]
        self.request_opts += [set_query_params(self.query_params)]
        req = Request('contact/v3/users/:user_id', 'PUT', [ACCESS_TOKEN_TYPE_TENANT],
                      self.body, output_class=UserUpdateResult, request_opts=self.request_opts)
        resp = req.do(conf)
        return resp


class UserCreateReqCall(object):
    def __init__(self, service, body, request_opts=None):
        # type: (UserService, User, List[Any]) -> None

        self.service = service
        self.body = body
        self.query_params = {}  # type: Dict[str, Any]

        if request_opts:
            self.request_opts = request_opts
        else:
            self.request_opts = []  # type: List[Any]

    def set_user_id_type(self, user_id_type):
        # type: (str) -> UserCreateReqCall
        self.query_params['user_id_type'] = user_id_type
        return self

    def set_department_id_type(self, department_id_type):
        # type: (str) -> UserCreateReqCall
        self.query_params['department_id_type'] = department_id_type
        return self

    def set_client_token(self, client_token):
        # type: (str) -> UserCreateReqCall
        self.query_params['client_token'] = client_token
        return self

    def do(self):
        # type: () -> Response[UserCreateResult]
        root_service = self.service.service

        conf = root_service.conf
        self.request_opts += [set_query_params(self.query_params)]
        req = Request('contact/v3/users', 'POST', [ACCESS_TOKEN_TYPE_TENANT],
                      self.body, output_class=UserCreateResult, request_opts=self.request_opts)
        resp = req.do(conf)
        return resp


class UserPatchReqCall(object):
    def __init__(self, service, body, request_opts=None):
        # type: (UserService, User, List[Any]) -> None

        self.service = service
        self.body = body
        self.path_params = {}   # type: Dict[str, Any]
        self.query_params = {}  # type: Dict[str, Any]

        if request_opts:
            self.request_opts = request_opts
        else:
            self.request_opts = []  # type: List[Any]

    def set_user_id(self, user_id):
        # type: (str) -> UserPatchReqCall
        self.path_params['user_id'] = user_id
        return self

    def set_user_id_type(self, user_id_type):
        # type: (str) -> UserPatchReqCall
        self.query_params['user_id_type'] = user_id_type
        return self

    def set_department_id_type(self, department_id_type):
        # type: (str) -> UserPatchReqCall
        self.query_params['department_id_type'] = department_id_type
        return self

    def do(self):
        # type: () -> Response[UserPatchResult]
        root_service = self.service.service

        conf = root_service.conf
        self.request_opts += [set_path_params(self.path_params)]
        self.request_opts += [set_query_params(self.query_params)]
        req = Request('contact/v3/users/:user_id', 'PATCH', [ACCESS_TOKEN_TYPE_TENANT, ACCESS_TOKEN_TYPE_USER],
                      self.body, output_class=UserPatchResult, request_opts=self.request_opts)
        resp = req.do(conf)
        return resp


class UserUpdateUserIdReqCall(object):
    def __init__(self, service, body, request_opts=None):
        # type: (UserService, UserUpdateUserIdReqBody, List[Any]) -> None

        self.service = service
        self.body = body
        self.path_params = {}   # type: Dict[str, Any]
        self.query_params = {}  # type: Dict[str, Any]

        if request_opts:
            self.request_opts = request_opts
        else:
            self.request_opts = []  # type: List[Any]

    def set_user_id(self, user_id):
        # type: (str) -> UserUpdateUserIdReqCall
        self.path_params['user_id'] = user_id
        return self

    def set_user_id_type(self, user_id_type):
        # type: (str) -> UserUpdateUserIdReqCall
        self.query_params['user_id_type'] = user_id_type
        return self

    def do(self):
        # type: () -> Response[None]
        root_service = self.service.service

        conf = root_service.conf
        self.request_opts += [set_path_params(self.path_params)]
        self.request_opts += [set_query_params(self.query_params)]
        req = Request('contact/v3/users/:user_id/update_user_id', 'PATCH', [ACCESS_TOKEN_TYPE_TENANT],
                      self.body, request_opts=self.request_opts)
        resp = req.do(conf)
        return resp


class UserGroupMemberCreateReqCall(object):
    def __init__(self, service, body, request_opts=None):
        # type: (UserGroupMemberService, UserGroupMemberCreateReqBody, List[Any]) -> None

        self.service = service
        self.body = body
        self.path_params = {}   # type: Dict[str, Any]
        self.query_params = {}  # type: Dict[str, Any]

        if request_opts:
            self.request_opts = request_opts
        else:
            self.request_opts = []  # type: List[Any]

    def set_user_group_id(self, user_group_id):
        # type: (str) -> UserGroupMemberCreateReqCall
        self.path_params['user_group_id'] = user_group_id
        return self

    def set_user_id_type(self, user_id_type):
        # type: (str) -> UserGroupMemberCreateReqCall
        self.query_params['user_id_type'] = user_id_type
        return self

    def do(self):
        # type: () -> Response[None]
        root_service = self.service.service

        conf = root_service.conf
        self.request_opts += [set_path_params(self.path_params)]
        self.request_opts += [set_query_params(self.query_params)]
        req = Request('contact/v3/user_groups/:user_group_id/members', 'POST', [ACCESS_TOKEN_TYPE_TENANT],
                      self.body, request_opts=self.request_opts)
        resp = req.do(conf)
        return resp


class UserGroupDeleteReqCall(object):
    def __init__(self, service, request_opts=None):
        # type: (UserGroupService, List[Any]) -> None

        self.service = service
        
        self.path_params = {}   # type: Dict[str, Any]

        if request_opts:
            self.request_opts = request_opts
        else:
            self.request_opts = []  # type: List[Any]

    def set_user_group_id(self, user_group_id):
        # type: (str) -> UserGroupDeleteReqCall
        self.path_params['user_group_id'] = user_group_id
        return self

    def do(self):
        # type: () -> Response[None]
        root_service = self.service.service

        conf = root_service.conf
        self.request_opts += [set_path_params(self.path_params)]
        req = Request('contact/v3/user_groups/:user_group_id', 'DELETE', [ACCESS_TOKEN_TYPE_TENANT],
                      None, request_opts=self.request_opts)
        resp = req.do(conf)
        return resp


class DepartmentCreateReqCall(object):
    def __init__(self, service, body, request_opts=None):
        # type: (DepartmentService, Department, List[Any]) -> None

        self.service = service
        self.body = body
        self.query_params = {}  # type: Dict[str, Any]

        if request_opts:
            self.request_opts = request_opts
        else:
            self.request_opts = []  # type: List[Any]

    def set_user_id_type(self, user_id_type):
        # type: (str) -> DepartmentCreateReqCall
        self.query_params['user_id_type'] = user_id_type
        return self

    def set_department_id_type(self, department_id_type):
        # type: (str) -> DepartmentCreateReqCall
        self.query_params['department_id_type'] = department_id_type
        return self

    def set_client_token(self, client_token):
        # type: (str) -> DepartmentCreateReqCall
        self.query_params['client_token'] = client_token
        return self

    def do(self):
        # type: () -> Response[DepartmentCreateResult]
        root_service = self.service.service

        conf = root_service.conf
        self.request_opts += [set_query_params(self.query_params)]
        req = Request('contact/v3/departments', 'POST', [ACCESS_TOKEN_TYPE_TENANT],
                      self.body, output_class=DepartmentCreateResult, request_opts=self.request_opts)
        resp = req.do(conf)
        return resp


class UserGroupPatchReqCall(object):
    def __init__(self, service, body, request_opts=None):
        # type: (UserGroupService, UserGroup, List[Any]) -> None

        self.service = service
        self.body = body
        self.path_params = {}   # type: Dict[str, Any]

        if request_opts:
            self.request_opts = request_opts
        else:
            self.request_opts = []  # type: List[Any]

    def set_user_group_id(self, user_group_id):
        # type: (str) -> UserGroupPatchReqCall
        self.path_params['user_group_id'] = user_group_id
        return self

    def do(self):
        # type: () -> Response[UserGroupPatchResult]
        root_service = self.service.service

        conf = root_service.conf
        self.request_opts += [set_path_params(self.path_params)]
        req = Request('contact/v3/user_groups/:user_group_id', 'PATCH', [ACCESS_TOKEN_TYPE_TENANT],
                      self.body, output_class=UserGroupPatchResult, request_opts=self.request_opts)
        resp = req.do(conf)
        return resp


class DepartmentUnitPatchReqCall(object):
    def __init__(self, service, body, request_opts=None):
        # type: (DepartmentUnitService, DepartmentUnitPatchReqBody, List[Any]) -> None

        self.service = service
        self.body = body
        self.path_params = {}   # type: Dict[str, Any]

        if request_opts:
            self.request_opts = request_opts
        else:
            self.request_opts = []  # type: List[Any]

    def set_unit_id(self, unit_id):
        # type: (str) -> DepartmentUnitPatchReqCall
        self.path_params['unit_id'] = unit_id
        return self

    def do(self):
        # type: () -> Response[DepartmentUnitPatchResult]
        root_service = self.service.service

        conf = root_service.conf
        self.request_opts += [set_path_params(self.path_params)]
        req = Request('contact/v3/department_units/:unit_id', 'PATCH', [ACCESS_TOKEN_TYPE_TENANT],
                      self.body, output_class=DepartmentUnitPatchResult, request_opts=self.request_opts)
        resp = req.do(conf)
        return resp


class UserGroupCreateReqCall(object):
    def __init__(self, service, body, request_opts=None):
        # type: (UserGroupService, UserGroup, List[Any]) -> None

        self.service = service
        self.body = body

        if request_opts:
            self.request_opts = request_opts
        else:
            self.request_opts = []  # type: List[Any]

    def do(self):
        # type: () -> Response[UserGroupCreateResult]
        root_service = self.service.service

        conf = root_service.conf
        req = Request('contact/v3/user_groups', 'POST', [ACCESS_TOKEN_TYPE_TENANT],
                      self.body, output_class=UserGroupCreateResult, request_opts=self.request_opts)
        resp = req.do(conf)
        return resp


class UserGroupListReqCall(object):
    def __init__(self, service, request_opts=None):
        # type: (UserGroupService, List[Any]) -> None

        self.service = service
        
        self.query_params = {}  # type: Dict[str, Any]

        if request_opts:
            self.request_opts = request_opts
        else:
            self.request_opts = []  # type: List[Any]

    def set_page_size(self, page_size):
        # type: (int) -> UserGroupListReqCall
        self.query_params['page_size'] = page_size
        return self

    def set_page_token(self, page_token):
        # type: (str) -> UserGroupListReqCall
        self.query_params['page_token'] = page_token
        return self

    def do(self):
        # type: () -> Response[UserGroupListResult]
        root_service = self.service.service

        conf = root_service.conf
        self.request_opts += [set_query_params(self.query_params)]
        req = Request('contact/v3/user_groups', 'GET', [ACCESS_TOKEN_TYPE_TENANT],
                      None, output_class=UserGroupListResult, request_opts=self.request_opts)
        resp = req.do(conf)
        return resp


class DepartmentDeleteReqCall(object):
    def __init__(self, service, request_opts=None):
        # type: (DepartmentService, List[Any]) -> None

        self.service = service
        
        self.path_params = {}   # type: Dict[str, Any]
        self.query_params = {}  # type: Dict[str, Any]

        if request_opts:
            self.request_opts = request_opts
        else:
            self.request_opts = []  # type: List[Any]

    def set_department_id(self, department_id):
        # type: (str) -> DepartmentDeleteReqCall
        self.path_params['department_id'] = department_id
        return self

    def set_user_id_type(self, user_id_type):
        # type: (str) -> DepartmentDeleteReqCall
        self.query_params['user_id_type'] = user_id_type
        return self

    def set_department_id_type(self, department_id_type):
        # type: (str) -> DepartmentDeleteReqCall
        self.query_params['department_id_type'] = department_id_type
        return self

    def do(self):
        # type: () -> Response[None]
        root_service = self.service.service

        conf = root_service.conf
        self.request_opts += [set_path_params(self.path_params)]
        self.request_opts += [set_query_params(self.query_params)]
        req = Request('contact/v3/departments/:department_id', 'DELETE', [ACCESS_TOKEN_TYPE_TENANT],
                      None, request_opts=self.request_opts)
        resp = req.do(conf)
        return resp


class UserGetReqCall(object):
    def __init__(self, service, request_opts=None):
        # type: (UserService, List[Any]) -> None

        self.service = service
        
        self.path_params = {}   # type: Dict[str, Any]
        self.query_params = {}  # type: Dict[str, Any]

        if request_opts:
            self.request_opts = request_opts
        else:
            self.request_opts = []  # type: List[Any]

    def set_user_id(self, user_id):
        # type: (str) -> UserGetReqCall
        self.path_params['user_id'] = user_id
        return self

    def set_user_id_type(self, user_id_type):
        # type: (str) -> UserGetReqCall
        self.query_params['user_id_type'] = user_id_type
        return self

    def set_department_id_type(self, department_id_type):
        # type: (str) -> UserGetReqCall
        self.query_params['department_id_type'] = department_id_type
        return self

    def do(self):
        # type: () -> Response[UserGetResult]
        root_service = self.service.service

        conf = root_service.conf
        self.request_opts += [set_path_params(self.path_params)]
        self.request_opts += [set_query_params(self.query_params)]
        req = Request('contact/v3/users/:user_id', 'GET', [ACCESS_TOKEN_TYPE_TENANT, ACCESS_TOKEN_TYPE_USER],
                      None, output_class=UserGetResult, request_opts=self.request_opts)
        resp = req.do(conf)
        return resp


class UserGroupMemberListReqCall(object):
    def __init__(self, service, request_opts=None):
        # type: (UserGroupMemberService, List[Any]) -> None

        self.service = service
        
        self.path_params = {}   # type: Dict[str, Any]
        self.query_params = {}  # type: Dict[str, Any]

        if request_opts:
            self.request_opts = request_opts
        else:
            self.request_opts = []  # type: List[Any]

    def set_user_group_id(self, user_group_id):
        # type: (str) -> UserGroupMemberListReqCall
        self.path_params['user_group_id'] = user_group_id
        return self

    def set_user_id_type(self, user_id_type):
        # type: (str) -> UserGroupMemberListReqCall
        self.query_params['user_id_type'] = user_id_type
        return self

    def set_department_id_type(self, department_id_type):
        # type: (str) -> UserGroupMemberListReqCall
        self.query_params['department_id_type'] = department_id_type
        return self

    def set_page_token(self, page_token):
        # type: (str) -> UserGroupMemberListReqCall
        self.query_params['page_token'] = page_token
        return self

    def set_page_size(self, page_size):
        # type: (int) -> UserGroupMemberListReqCall
        self.query_params['page_size'] = page_size
        return self

    def do(self):
        # type: () -> Response[UserGroupMemberListResult]
        root_service = self.service.service

        conf = root_service.conf
        self.request_opts += [set_path_params(self.path_params)]
        self.request_opts += [set_query_params(self.query_params)]
        req = Request('contact/v3/user_groups/:user_group_id/members', 'GET', [ACCESS_TOKEN_TYPE_TENANT],
                      None, output_class=UserGroupMemberListResult, request_opts=self.request_opts)
        resp = req.do(conf)
        return resp


class UserGroupMemberDeleteReqCall(object):
    def __init__(self, service, request_opts=None):
        # type: (UserGroupMemberService, List[Any]) -> None

        self.service = service
        
        self.path_params = {}   # type: Dict[str, Any]
        self.query_params = {}  # type: Dict[str, Any]

        if request_opts:
            self.request_opts = request_opts
        else:
            self.request_opts = []  # type: List[Any]

    def set_user_group_id(self, user_group_id):
        # type: (str) -> UserGroupMemberDeleteReqCall
        self.path_params['user_group_id'] = user_group_id
        return self

    def set_user_id(self, user_id):
        # type: (str) -> UserGroupMemberDeleteReqCall
        self.path_params['user_id'] = user_id
        return self

    def set_user_id_type(self, user_id_type):
        # type: (str) -> UserGroupMemberDeleteReqCall
        self.query_params['user_id_type'] = user_id_type
        return self

    def do(self):
        # type: () -> Response[None]
        root_service = self.service.service

        conf = root_service.conf
        self.request_opts += [set_path_params(self.path_params)]
        self.request_opts += [set_query_params(self.query_params)]
        req = Request('contact/v3/user_groups/:user_group_id/members/:user_id', 'DELETE', [ACCESS_TOKEN_TYPE_TENANT],
                      None, request_opts=self.request_opts)
        resp = req.do(conf)
        return resp


class DepartmentUpdateReqCall(object):
    def __init__(self, service, body, request_opts=None):
        # type: (DepartmentService, Department, List[Any]) -> None

        self.service = service
        self.body = body
        self.path_params = {}   # type: Dict[str, Any]
        self.query_params = {}  # type: Dict[str, Any]

        if request_opts:
            self.request_opts = request_opts
        else:
            self.request_opts = []  # type: List[Any]

    def set_department_id(self, department_id):
        # type: (str) -> DepartmentUpdateReqCall
        self.path_params['department_id'] = department_id
        return self

    def set_user_id_type(self, user_id_type):
        # type: (str) -> DepartmentUpdateReqCall
        self.query_params['user_id_type'] = user_id_type
        return self

    def set_department_id_type(self, department_id_type):
        # type: (str) -> DepartmentUpdateReqCall
        self.query_params['department_id_type'] = department_id_type
        return self

    def do(self):
        # type: () -> Response[DepartmentUpdateResult]
        root_service = self.service.service

        conf = root_service.conf
        self.request_opts += [set_path_params(self.path_params)]
        self.request_opts += [set_query_params(self.query_params)]
        req = Request('contact/v3/departments/:department_id', 'PUT', [ACCESS_TOKEN_TYPE_TENANT],
                      self.body, output_class=DepartmentUpdateResult, request_opts=self.request_opts)
        resp = req.do(conf)
        return resp


class DepartmentUnitCreateReqCall(object):
    def __init__(self, service, body, request_opts=None):
        # type: (DepartmentUnitService, DepartmentUnit, List[Any]) -> None

        self.service = service
        self.body = body

        if request_opts:
            self.request_opts = request_opts
        else:
            self.request_opts = []  # type: List[Any]

    def do(self):
        # type: () -> Response[DepartmentUnitCreateResult]
        root_service = self.service.service

        conf = root_service.conf
        req = Request('contact/v3/department_units', 'POST', [ACCESS_TOKEN_TYPE_TENANT],
                      self.body, output_class=DepartmentUnitCreateResult, request_opts=self.request_opts)
        resp = req.do(conf)
        return resp


class UserListReqCall(object):
    def __init__(self, service, request_opts=None):
        # type: (UserService, List[Any]) -> None

        self.service = service
        
        self.query_params = {}  # type: Dict[str, Any]

        if request_opts:
            self.request_opts = request_opts
        else:
            self.request_opts = []  # type: List[Any]

    def set_user_id_type(self, user_id_type):
        # type: (str) -> UserListReqCall
        self.query_params['user_id_type'] = user_id_type
        return self

    def set_department_id_type(self, department_id_type):
        # type: (str) -> UserListReqCall
        self.query_params['department_id_type'] = department_id_type
        return self

    def set_department_id(self, department_id):
        # type: (str) -> UserListReqCall
        self.query_params['department_id'] = department_id
        return self

    def set_page_token(self, page_token):
        # type: (str) -> UserListReqCall
        self.query_params['page_token'] = page_token
        return self

    def set_page_size(self, page_size):
        # type: (int) -> UserListReqCall
        self.query_params['page_size'] = page_size
        return self

    def do(self):
        # type: () -> Response[UserListResult]
        root_service = self.service.service

        conf = root_service.conf
        self.request_opts += [set_query_params(self.query_params)]
        req = Request('contact/v3/users', 'GET', [ACCESS_TOKEN_TYPE_TENANT, ACCESS_TOKEN_TYPE_USER],
                      None, output_class=UserListResult, request_opts=self.request_opts)
        resp = req.do(conf)
        return resp


class UserGroupGetReqCall(object):
    def __init__(self, service, request_opts=None):
        # type: (UserGroupService, List[Any]) -> None

        self.service = service
        
        self.path_params = {}   # type: Dict[str, Any]

        if request_opts:
            self.request_opts = request_opts
        else:
            self.request_opts = []  # type: List[Any]

    def set_user_group_id(self, user_group_id):
        # type: (str) -> UserGroupGetReqCall
        self.path_params['user_group_id'] = user_group_id
        return self

    def do(self):
        # type: () -> Response[UserGroupGetResult]
        root_service = self.service.service

        conf = root_service.conf
        self.request_opts += [set_path_params(self.path_params)]
        req = Request('contact/v3/user_groups/:user_group_id', 'GET', [ACCESS_TOKEN_TYPE_TENANT],
                      None, output_class=UserGroupGetResult, request_opts=self.request_opts)
        resp = req.do(conf)
        return resp


class DepartmentParentReqCall(object):
    def __init__(self, service, request_opts=None):
        # type: (DepartmentService, List[Any]) -> None

        self.service = service
        
        self.query_params = {}  # type: Dict[str, Any]

        if request_opts:
            self.request_opts = request_opts
        else:
            self.request_opts = []  # type: List[Any]

    def set_user_id_type(self, user_id_type):
        # type: (str) -> DepartmentParentReqCall
        self.query_params['user_id_type'] = user_id_type
        return self

    def set_department_id_type(self, department_id_type):
        # type: (str) -> DepartmentParentReqCall
        self.query_params['department_id_type'] = department_id_type
        return self

    def set_department_id(self, department_id):
        # type: (str) -> DepartmentParentReqCall
        self.query_params['department_id'] = department_id
        return self

    def set_page_token(self, page_token):
        # type: (str) -> DepartmentParentReqCall
        self.query_params['page_token'] = page_token
        return self

    def set_page_size(self, page_size):
        # type: (int) -> DepartmentParentReqCall
        self.query_params['page_size'] = page_size
        return self

    def do(self):
        # type: () -> Response[DepartmentParentResult]
        root_service = self.service.service

        conf = root_service.conf
        self.request_opts += [set_query_params(self.query_params)]
        req = Request('contact/v3/departments/parent', 'GET', [ACCESS_TOKEN_TYPE_TENANT, ACCESS_TOKEN_TYPE_USER],
                      None, output_class=DepartmentParentResult, request_opts=self.request_opts)
        resp = req.do(conf)
        return resp


class DepartmentUpdateDepartmentIdReqCall(object):
    def __init__(self, service, body, request_opts=None):
        # type: (DepartmentService, DepartmentUpdateDepartmentIdReqBody, List[Any]) -> None

        self.service = service
        self.body = body
        self.path_params = {}   # type: Dict[str, Any]
        self.query_params = {}  # type: Dict[str, Any]

        if request_opts:
            self.request_opts = request_opts
        else:
            self.request_opts = []  # type: List[Any]

    def set_department_id(self, department_id):
        # type: (str) -> DepartmentUpdateDepartmentIdReqCall
        self.path_params['department_id'] = department_id
        return self

    def set_department_id_type(self, department_id_type):
        # type: (str) -> DepartmentUpdateDepartmentIdReqCall
        self.query_params['department_id_type'] = department_id_type
        return self

    def do(self):
        # type: () -> Response[None]
        root_service = self.service.service

        conf = root_service.conf
        self.request_opts += [set_path_params(self.path_params)]
        self.request_opts += [set_query_params(self.query_params)]
        req = Request('contact/v3/departments/:department_id/update_department_id', 'PATCH', [ACCESS_TOKEN_TYPE_TENANT],
                      self.body, request_opts=self.request_opts)
        resp = req.do(conf)
        return resp


class DepartmentUnbindDepartmentChatReqCall(object):
    def __init__(self, service, body, request_opts=None):
        # type: (DepartmentService, DepartmentUnbindDepartmentChatReqBody, List[Any]) -> None

        self.service = service
        self.body = body
        self.query_params = {}  # type: Dict[str, Any]

        if request_opts:
            self.request_opts = request_opts
        else:
            self.request_opts = []  # type: List[Any]

    def set_department_id_type(self, department_id_type):
        # type: (str) -> DepartmentUnbindDepartmentChatReqCall
        self.query_params['department_id_type'] = department_id_type
        return self

    def do(self):
        # type: () -> Response[None]
        root_service = self.service.service

        conf = root_service.conf
        self.request_opts += [set_query_params(self.query_params)]
        req = Request('contact/v3/departments/unbind_department_chat', 'POST', [ACCESS_TOKEN_TYPE_TENANT],
                      self.body, request_opts=self.request_opts)
        resp = req.do(conf)
        return resp


class DepartmentSearchReqCall(object):
    def __init__(self, service, body, request_opts=None):
        # type: (DepartmentService, DepartmentSearchReqBody, List[Any]) -> None

        self.service = service
        self.body = body
        self.query_params = {}  # type: Dict[str, Any]

        if request_opts:
            self.request_opts = request_opts
        else:
            self.request_opts = []  # type: List[Any]

    def set_user_id_type(self, user_id_type):
        # type: (str) -> DepartmentSearchReqCall
        self.query_params['user_id_type'] = user_id_type
        return self

    def set_department_id_type(self, department_id_type):
        # type: (str) -> DepartmentSearchReqCall
        self.query_params['department_id_type'] = department_id_type
        return self

    def set_page_token(self, page_token):
        # type: (str) -> DepartmentSearchReqCall
        self.query_params['page_token'] = page_token
        return self

    def set_page_size(self, page_size):
        # type: (int) -> DepartmentSearchReqCall
        self.query_params['page_size'] = page_size
        return self

    def do(self):
        # type: () -> Response[DepartmentSearchResult]
        root_service = self.service.service

        conf = root_service.conf
        self.request_opts += [set_query_params(self.query_params)]
        req = Request('contact/v3/departments/search', 'POST', [ACCESS_TOKEN_TYPE_USER],
                      self.body, output_class=DepartmentSearchResult, request_opts=self.request_opts)
        resp = req.do(conf)
        return resp


class EmployeeTypeEnumListReqCall(object):
    def __init__(self, service, request_opts=None):
        # type: (EmployeeTypeEnumService, List[Any]) -> None

        self.service = service
        
        self.query_params = {}  # type: Dict[str, Any]

        if request_opts:
            self.request_opts = request_opts
        else:
            self.request_opts = []  # type: List[Any]

    def set_page_token(self, page_token):
        # type: (str) -> EmployeeTypeEnumListReqCall
        self.query_params['page_token'] = page_token
        return self

    def set_page_size(self, page_size):
        # type: (int) -> EmployeeTypeEnumListReqCall
        self.query_params['page_size'] = page_size
        return self

    def do(self):
        # type: () -> Response[EmployeeTypeEnumListResult]
        root_service = self.service.service

        conf = root_service.conf
        self.request_opts += [set_query_params(self.query_params)]
        req = Request('contact/v3/employee_type_enums', 'GET', [ACCESS_TOKEN_TYPE_TENANT],
                      None, output_class=EmployeeTypeEnumListResult, request_opts=self.request_opts)
        resp = req.do(conf)
        return resp




# -*- coding: UTF-8 -*-
# Code generated by lark suite oapi sdk gen

from typing import List, Dict, Any
from ....utils.dt import to_json_decorator
import attr




@to_json_decorator
@attr.s
class Collaborator(object):
    member_type = attr.ib(type=str, default=None, metadata={'json': 'member_type'})
    member_open_id = attr.ib(type=str, default=None, metadata={'json': 'member_open_id'})
    member_user_id = attr.ib(type=str, default=None, metadata={'json': 'member_user_id'})
    perm = attr.ib(type=str, default=None, metadata={'json': 'perm'})


@to_json_decorator
@attr.s
class Member(object):
    member_type = attr.ib(type=str, default=None, metadata={'json': 'member_type'})
    member_id = attr.ib(type=str, default=None, metadata={'json': 'member_id'})
    perm = attr.ib(type=str, default=None, metadata={'json': 'perm'})


@to_json_decorator
@attr.s
class Owner(object):
    member_type = attr.ib(type=str, default=None, metadata={'json': 'member_type'})
    member_id = attr.ib(type=str, default=None, metadata={'json': 'member_id'})


@to_json_decorator
@attr.s
class Public(object):
    pass



@to_json_decorator
@attr.s
class MemberCreateReqBody(object):
    token = attr.ib(type=str, default=None, metadata={'json': 'token'})
    type = attr.ib(type=str, default=None, metadata={'json': 'type'})
    members = attr.ib(type=List[Member], default=None, metadata={'json': 'members'})
    notify_lark = attr.ib(type=bool, default=None, metadata={'json': 'notify_lark'})


@attr.s
class MemberCreateResult(object):
    is_all_success = attr.ib(type=bool, default=None, metadata={'json': 'is_all_success'})
    fail_members = attr.ib(type=List[Member], default=None, metadata={'json': 'fail_members'})


@to_json_decorator
@attr.s
class MemberDeleteReqBody(object):
    token = attr.ib(type=str, default=None, metadata={'json': 'token'})
    type = attr.ib(type=str, default=None, metadata={'json': 'type'})
    member_type = attr.ib(type=str, default=None, metadata={'json': 'member_type'})


@attr.s
class MemberDeleteResult(object):
    is_success = attr.ib(type=bool, default=None, metadata={'json': 'is_success'})


@to_json_decorator
@attr.s
class MemberListReqBody(object):
    token = attr.ib(type=str, default=None, metadata={'json': 'token'})
    type = attr.ib(type=str, default=None, metadata={'json': 'type'})


@attr.s
class MemberListResult(object):
    members = attr.ib(type=List[Collaborator], default=None, metadata={'json': 'members'})


@to_json_decorator
@attr.s
class MemberPermittedReqBody(object):
    token = attr.ib(type=str, default=None, metadata={'json': 'token'})
    type = attr.ib(type=str, default=None, metadata={'json': 'type'})
    perm = attr.ib(type=str, default=None, metadata={'json': 'perm'})


@attr.s
class MemberPermittedResult(object):
    is_permitted = attr.ib(type=bool, default=None, metadata={'json': 'is_permitted'})


@to_json_decorator
@attr.s
class MemberTransferReqBody(object):
    token = attr.ib(type=str, default=None, metadata={'json': 'token'})
    type = attr.ib(type=str, default=None, metadata={'json': 'type'})
    owner = attr.ib(type=Owner, default=None, metadata={'json': 'owner'})
    remove_old_owner = attr.ib(type=bool, default=None, metadata={'json': 'remove_old_owner'})
    cancel_notify = attr.ib(type=bool, default=None, metadata={'json': 'cancel_notify'})


@attr.s
class MemberTransferResult(object):
    is_success = attr.ib(type=bool, default=None, metadata={'json': 'is_success'})
    type = attr.ib(type=str, default=None, metadata={'json': 'type'})
    token = attr.ib(type=str, default=None, metadata={'json': 'token'})
    owner = attr.ib(type=Owner, default=None, metadata={'json': 'owner'})


@to_json_decorator
@attr.s
class MemberUpdateReqBody(object):
    token = attr.ib(type=str, default=None, metadata={'json': 'token'})
    type = attr.ib(type=str, default=None, metadata={'json': 'type'})
    member_type = attr.ib(type=str, default=None, metadata={'json': 'member_type'})
    perm = attr.ib(type=str, default=None, metadata={'json': 'perm'})
    notify_lark = attr.ib(type=bool, default=None, metadata={'json': 'notify_lark'})


@attr.s
class MemberUpdateResult(object):
    is_success = attr.ib(type=bool, default=None, metadata={'json': 'is_success'})


@to_json_decorator
@attr.s
class PublicUpdateReqBody(object):
    token = attr.ib(type=str, default=None, metadata={'json': 'token'})
    type = attr.ib(type=str, default=None, metadata={'json': 'type'})
    copy_print_export_status = attr.ib(type=bool, default=None, metadata={'json': 'copy_print_export_status'})
    comment = attr.ib(type=bool, default=None, metadata={'json': 'comment'})
    tenant_shareable = attr.ib(type=bool, default=None, metadata={'json': 'tenant_shareable'})
    link_share_entity = attr.ib(type=str, default=None, metadata={'json': 'link_share_entity'})
    external_access = attr.ib(type=bool, default=None, metadata={'json': 'external_access'})
    invite_external = attr.ib(type=bool, default=None, metadata={'json': 'invite_external'})


@attr.s
class PublicUpdateResult(object):
    is_success = attr.ib(type=bool, default=None, metadata={'json': 'is_success'})


# -*- coding: UTF-8 -*-

import os
from typing import Tuple

from .consts import APP_TYPE_ISV, APP_TYPE_INTERNAL
from .logger import Logger, LoggerProxy
from .app_settings import AppSettings
from .store import MemoryStore, Store


class Config(object):

    def __init__(self, domain, app_settings, logger, log_level, store):
        # type: (str, AppSettings, Logger, int, Store) -> None
        self.domain = domain
        self.app_settings = app_settings
        self.logger = LoggerProxy(log_level, logger)
        self.store = store

    @staticmethod
    def new_config_with_memory_store(domain, settings, logger,
                                     log_level):  # type: (str, AppSettings, Logger, int) -> Config
        return Config(domain, settings, logger, log_level, MemoryStore())

    @staticmethod
    def new_config(domain, settings, logger, log_level,
                   store):  # type: (str, AppSettings, Logger, int, Store) -> Config
        return Config(domain, settings, logger, log_level, store)

    @staticmethod
    def __app_settings_from_env():  # type: () -> Tuple[str, str, str, str]
        """
        This function reads app_id, app_secret, ... from system env.
        If app_id or app_secret doesn't exist in system env, a RuntimeError will be raised.
        """

        app_id = os.getenv('APP_ID', '')
        app_secret = os.getenv('APP_SECRET', '')
        verification_token = os.getenv('VERIFICATION_TOKEN', '')
        encrypt_key = os.getenv('ENCRYPT_KEY', '')

        if app_id == '':
            raise RuntimeError('environment variables not exist `APP_ID`')
        if encrypt_key == '':
            raise RuntimeError('environment variables not exist `APP_SECRET`')

        return app_id, app_secret, verification_token, encrypt_key

    @staticmethod
    def new_isv_app_settings_from_env():
        """
        This static method initialize a new ISV App Settings and read app_id, app_secret, ... from system env
        """

        app_id, app_secret, verification_token, encrypt_key = Config.__app_settings_from_env()
        return AppSettings(APP_TYPE_ISV, app_id, app_secret, verification_token, encrypt_key)

    @staticmethod
    def new_internal_app_settings_from_env():
        """
        This static method initialize a new internal App Settings and read app_id, app_secret, ... from system env
        """

        app_id, app_secret, verification_token, encrypt_key = Config.__app_settings_from_env()
        return AppSettings(APP_TYPE_INTERNAL, app_id, app_secret, verification_token, encrypt_key)

    @staticmethod
    def new_internal_app_settings(app_id, app_secret, verification_token,
                                  encrypt_key):  # type: (str, str, str, str) -> AppSettings
        return AppSettings(APP_TYPE_INTERNAL, app_id, app_secret, verification_token, encrypt_key)

    @staticmethod
    def new_isv_app_settings(app_id, app_secret, verification_token,
                             encrypt_key):  # type: (str, str, str, str) -> AppSettings
        return AppSettings(APP_TYPE_ISV, app_id, app_secret, verification_token, encrypt_key)



# -*- coding: UTF-8 -*-
# Code generated by lark suite oapi sdk gen

from typing import *

from ....api import Request, Response, set_timeout, set_tenant_key, set_user_access_token, set_path_params, \
    set_query_params, set_response_stream, set_is_response_stream, FormData, FormDataFile
from ....config import Config
from ....consts import ACCESS_TOKEN_TYPE_TENANT, ACCESS_TOKEN_TYPE_USER, ACCESS_TOKEN_TYPE_APP
from .model import *


class Service(object):
    def __init__(self, conf):
        # type: (Config) -> None
        self.conf = conf
        self.calendars = CalendarService(self)
        self.calendar_acls = CalendarAclService(self)
        self.calendar_events = CalendarEventService(self)
        self.calendar_event_attendees = CalendarEventAttendeeService(self)
        self.freebusys = FreebusyService(self)
        self.timeoff_events = TimeoffEventService(self)
        self.settings = SettingService(self)
        self.calendar_event_attendee_chat_members = CalendarEventAttendeeChatMemberService(self)
        



class CalendarService(object):
    def __init__(self, service):
        # type: (Service) -> None
        self.service = service

    def create(self, body, tenant_key=None, user_access_token=None, timeout=None):
        # type: (Calendar, str, str, int) -> CalendarCreateReqCall

        request_opts = []   # type: List[Callable[[Any], Any]]

        if timeout is not None:
            request_opts += [set_timeout(timeout)]

        if tenant_key is not None:
            request_opts += [set_tenant_key(tenant_key)]

        if user_access_token is not None:
            request_opts += [set_user_access_token(user_access_token)]

        return CalendarCreateReqCall(self, body, request_opts=request_opts)

    def patch(self, body, tenant_key=None, user_access_token=None, timeout=None):
        # type: (Calendar, str, str, int) -> CalendarPatchReqCall

        request_opts = []   # type: List[Callable[[Any], Any]]

        if timeout is not None:
            request_opts += [set_timeout(timeout)]

        if tenant_key is not None:
            request_opts += [set_tenant_key(tenant_key)]

        if user_access_token is not None:
            request_opts += [set_user_access_token(user_access_token)]

        return CalendarPatchReqCall(self, body, request_opts=request_opts)

    def delete(self, tenant_key=None, user_access_token=None, timeout=None):
        # type: (str, str, int) -> CalendarDeleteReqCall

        request_opts = []   # type: List[Callable[[Any], Any]]

        if timeout is not None:
            request_opts += [set_timeout(timeout)]

        if tenant_key is not None:
            request_opts += [set_tenant_key(tenant_key)]

        if user_access_token is not None:
            request_opts += [set_user_access_token(user_access_token)]

        return CalendarDeleteReqCall(self, request_opts=request_opts)

    def list(self, tenant_key=None, user_access_token=None, timeout=None):
        # type: (str, str, int) -> CalendarListReqCall

        request_opts = []   # type: List[Callable[[Any], Any]]

        if timeout is not None:
            request_opts += [set_timeout(timeout)]

        if tenant_key is not None:
            request_opts += [set_tenant_key(tenant_key)]

        if user_access_token is not None:
            request_opts += [set_user_access_token(user_access_token)]

        return CalendarListReqCall(self, request_opts=request_opts)

    def get(self, tenant_key=None, user_access_token=None, timeout=None):
        # type: (str, str, int) -> CalendarGetReqCall

        request_opts = []   # type: List[Callable[[Any], Any]]

        if timeout is not None:
            request_opts += [set_timeout(timeout)]

        if tenant_key is not None:
            request_opts += [set_tenant_key(tenant_key)]

        if user_access_token is not None:
            request_opts += [set_user_access_token(user_access_token)]

        return CalendarGetReqCall(self, request_opts=request_opts)

    def search(self, body, tenant_key=None, timeout=None):
        # type: (CalendarSearchReqBody, str, int) -> CalendarSearchReqCall

        request_opts = []   # type: List[Callable[[Any], Any]]

        if timeout is not None:
            request_opts += [set_timeout(timeout)]

        if tenant_key is not None:
            request_opts += [set_tenant_key(tenant_key)]

        return CalendarSearchReqCall(self, body, request_opts=request_opts)

    def unsubscribe(self, tenant_key=None, user_access_token=None, timeout=None):
        # type: (str, str, int) -> CalendarUnsubscribeReqCall

        request_opts = []   # type: List[Callable[[Any], Any]]

        if timeout is not None:
            request_opts += [set_timeout(timeout)]

        if tenant_key is not None:
            request_opts += [set_tenant_key(tenant_key)]

        if user_access_token is not None:
            request_opts += [set_user_access_token(user_access_token)]

        return CalendarUnsubscribeReqCall(self, request_opts=request_opts)

    def subscribe(self, tenant_key=None, user_access_token=None, timeout=None):
        # type: (str, str, int) -> CalendarSubscribeReqCall

        request_opts = []   # type: List[Callable[[Any], Any]]

        if timeout is not None:
            request_opts += [set_timeout(timeout)]

        if tenant_key is not None:
            request_opts += [set_tenant_key(tenant_key)]

        if user_access_token is not None:
            request_opts += [set_user_access_token(user_access_token)]

        return CalendarSubscribeReqCall(self, request_opts=request_opts)

    def subscription(self, user_access_token=None, timeout=None):
        # type: (str, int) -> CalendarSubscriptionReqCall

        request_opts = []   # type: List[Callable[[Any], Any]]

        if timeout is not None:
            request_opts += [set_timeout(timeout)]

        if user_access_token is not None:
            request_opts += [set_user_access_token(user_access_token)]

        return CalendarSubscriptionReqCall(self, request_opts=request_opts)


class CalendarAclService(object):
    def __init__(self, service):
        # type: (Service) -> None
        self.service = service

    def list(self, tenant_key=None, user_access_token=None, timeout=None):
        # type: (str, str, int) -> CalendarAclListReqCall

        request_opts = []   # type: List[Callable[[Any], Any]]

        if timeout is not None:
            request_opts += [set_timeout(timeout)]

        if tenant_key is not None:
            request_opts += [set_tenant_key(tenant_key)]

        if user_access_token is not None:
            request_opts += [set_user_access_token(user_access_token)]

        return CalendarAclListReqCall(self, request_opts=request_opts)

    def delete(self, tenant_key=None, user_access_token=None, timeout=None):
        # type: (str, str, int) -> CalendarAclDeleteReqCall

        request_opts = []   # type: List[Callable[[Any], Any]]

        if timeout is not None:
            request_opts += [set_timeout(timeout)]

        if tenant_key is not None:
            request_opts += [set_tenant_key(tenant_key)]

        if user_access_token is not None:
            request_opts += [set_user_access_token(user_access_token)]

        return CalendarAclDeleteReqCall(self, request_opts=request_opts)

    def create(self, body, tenant_key=None, user_access_token=None, timeout=None):
        # type: (CalendarAcl, str, str, int) -> CalendarAclCreateReqCall

        request_opts = []   # type: List[Callable[[Any], Any]]

        if timeout is not None:
            request_opts += [set_timeout(timeout)]

        if tenant_key is not None:
            request_opts += [set_tenant_key(tenant_key)]

        if user_access_token is not None:
            request_opts += [set_user_access_token(user_access_token)]

        return CalendarAclCreateReqCall(self, body, request_opts=request_opts)

    def subscription(self, user_access_token=None, timeout=None):
        # type: (str, int) -> CalendarAclSubscriptionReqCall

        request_opts = []   # type: List[Callable[[Any], Any]]

        if timeout is not None:
            request_opts += [set_timeout(timeout)]

        if user_access_token is not None:
            request_opts += [set_user_access_token(user_access_token)]

        return CalendarAclSubscriptionReqCall(self, request_opts=request_opts)


class CalendarEventService(object):
    def __init__(self, service):
        # type: (Service) -> None
        self.service = service

    def delete(self, tenant_key=None, user_access_token=None, timeout=None):
        # type: (str, str, int) -> CalendarEventDeleteReqCall

        request_opts = []   # type: List[Callable[[Any], Any]]

        if timeout is not None:
            request_opts += [set_timeout(timeout)]

        if tenant_key is not None:
            request_opts += [set_tenant_key(tenant_key)]

        if user_access_token is not None:
            request_opts += [set_user_access_token(user_access_token)]

        return CalendarEventDeleteReqCall(self, request_opts=request_opts)

    def get(self, tenant_key=None, user_access_token=None, timeout=None):
        # type: (str, str, int) -> CalendarEventGetReqCall

        request_opts = []   # type: List[Callable[[Any], Any]]

        if timeout is not None:
            request_opts += [set_timeout(timeout)]

        if tenant_key is not None:
            request_opts += [set_tenant_key(tenant_key)]

        if user_access_token is not None:
            request_opts += [set_user_access_token(user_access_token)]

        return CalendarEventGetReqCall(self, request_opts=request_opts)

    def create(self, body, tenant_key=None, user_access_token=None, timeout=None):
        # type: (CalendarEvent, str, str, int) -> CalendarEventCreateReqCall

        request_opts = []   # type: List[Callable[[Any], Any]]

        if timeout is not None:
            request_opts += [set_timeout(timeout)]

        if tenant_key is not None:
            request_opts += [set_tenant_key(tenant_key)]

        if user_access_token is not None:
            request_opts += [set_user_access_token(user_access_token)]

        return CalendarEventCreateReqCall(self, body, request_opts=request_opts)

    def list(self, tenant_key=None, user_access_token=None, timeout=None):
        # type: (str, str, int) -> CalendarEventListReqCall

        request_opts = []   # type: List[Callable[[Any], Any]]

        if timeout is not None:
            request_opts += [set_timeout(timeout)]

        if tenant_key is not None:
            request_opts += [set_tenant_key(tenant_key)]

        if user_access_token is not None:
            request_opts += [set_user_access_token(user_access_token)]

        return CalendarEventListReqCall(self, request_opts=request_opts)

    def patch(self, body, tenant_key=None, user_access_token=None, timeout=None):
        # type: (CalendarEvent, str, str, int) -> CalendarEventPatchReqCall

        request_opts = []   # type: List[Callable[[Any], Any]]

        if timeout is not None:
            request_opts += [set_timeout(timeout)]

        if tenant_key is not None:
            request_opts += [set_tenant_key(tenant_key)]

        if user_access_token is not None:
            request_opts += [set_user_access_token(user_access_token)]

        return CalendarEventPatchReqCall(self, body, request_opts=request_opts)

    def search(self, body, user_access_token=None, timeout=None):
        # type: (CalendarEventSearchReqBody, str, int) -> CalendarEventSearchReqCall

        request_opts = []   # type: List[Callable[[Any], Any]]

        if timeout is not None:
            request_opts += [set_timeout(timeout)]

        if user_access_token is not None:
            request_opts += [set_user_access_token(user_access_token)]

        return CalendarEventSearchReqCall(self, body, request_opts=request_opts)

    def subscription(self, user_access_token=None, timeout=None):
        # type: (str, int) -> CalendarEventSubscriptionReqCall

        request_opts = []   # type: List[Callable[[Any], Any]]

        if timeout is not None:
            request_opts += [set_timeout(timeout)]

        if user_access_token is not None:
            request_opts += [set_user_access_token(user_access_token)]

        return CalendarEventSubscriptionReqCall(self, request_opts=request_opts)


class CalendarEventAttendeeService(object):
    def __init__(self, service):
        # type: (Service) -> None
        self.service = service

    def list(self, tenant_key=None, user_access_token=None, timeout=None):
        # type: (str, str, int) -> CalendarEventAttendeeListReqCall

        request_opts = []   # type: List[Callable[[Any], Any]]

        if timeout is not None:
            request_opts += [set_timeout(timeout)]

        if tenant_key is not None:
            request_opts += [set_tenant_key(tenant_key)]

        if user_access_token is not None:
            request_opts += [set_user_access_token(user_access_token)]

        return CalendarEventAttendeeListReqCall(self, request_opts=request_opts)

    def batch_delete(self, body, tenant_key=None, user_access_token=None, timeout=None):
        # type: (CalendarEventAttendeeBatchDeleteReqBody, str, str, int) -> CalendarEventAttendeeBatchDeleteReqCall

        request_opts = []   # type: List[Callable[[Any], Any]]

        if timeout is not None:
            request_opts += [set_timeout(timeout)]

        if tenant_key is not None:
            request_opts += [set_tenant_key(tenant_key)]

        if user_access_token is not None:
            request_opts += [set_user_access_token(user_access_token)]

        return CalendarEventAttendeeBatchDeleteReqCall(self, body, request_opts=request_opts)

    def create(self, body, tenant_key=None, user_access_token=None, timeout=None):
        # type: (CalendarEventAttendeeCreateReqBody, str, str, int) -> CalendarEventAttendeeCreateReqCall

        request_opts = []   # type: List[Callable[[Any], Any]]

        if timeout is not None:
            request_opts += [set_timeout(timeout)]

        if tenant_key is not None:
            request_opts += [set_tenant_key(tenant_key)]

        if user_access_token is not None:
            request_opts += [set_user_access_token(user_access_token)]

        return CalendarEventAttendeeCreateReqCall(self, body, request_opts=request_opts)


class FreebusyService(object):
    def __init__(self, service):
        # type: (Service) -> None
        self.service = service

    def list(self, body, tenant_key=None, timeout=None):
        # type: (FreebusyListReqBody, str, int) -> FreebusyListReqCall

        request_opts = []   # type: List[Callable[[Any], Any]]

        if timeout is not None:
            request_opts += [set_timeout(timeout)]

        if tenant_key is not None:
            request_opts += [set_tenant_key(tenant_key)]

        return FreebusyListReqCall(self, body, request_opts=request_opts)


class TimeoffEventService(object):
    def __init__(self, service):
        # type: (Service) -> None
        self.service = service

    def delete(self, tenant_key=None, timeout=None):
        # type: (str, int) -> TimeoffEventDeleteReqCall

        request_opts = []   # type: List[Callable[[Any], Any]]

        if timeout is not None:
            request_opts += [set_timeout(timeout)]

        if tenant_key is not None:
            request_opts += [set_tenant_key(tenant_key)]

        return TimeoffEventDeleteReqCall(self, request_opts=request_opts)

    def create(self, body, tenant_key=None, timeout=None):
        # type: (TimeoffEvent, str, int) -> TimeoffEventCreateReqCall

        request_opts = []   # type: List[Callable[[Any], Any]]

        if timeout is not None:
            request_opts += [set_timeout(timeout)]

        if tenant_key is not None:
            request_opts += [set_tenant_key(tenant_key)]

        return TimeoffEventCreateReqCall(self, body, request_opts=request_opts)


class SettingService(object):
    def __init__(self, service):
        # type: (Service) -> None
        self.service = service

    def generate_caldav_conf(self, body, user_access_token=None, timeout=None):
        # type: (SettingGenerateCaldavConfReqBody, str, int) -> SettingGenerateCaldavConfReqCall

        request_opts = []   # type: List[Callable[[Any], Any]]

        if timeout is not None:
            request_opts += [set_timeout(timeout)]

        if user_access_token is not None:
            request_opts += [set_user_access_token(user_access_token)]

        return SettingGenerateCaldavConfReqCall(self, body, request_opts=request_opts)


class CalendarEventAttendeeChatMemberService(object):
    def __init__(self, service):
        # type: (Service) -> None
        self.service = service

    def list(self, tenant_key=None, user_access_token=None, timeout=None):
        # type: (str, str, int) -> CalendarEventAttendeeChatMemberListReqCall

        request_opts = []   # type: List[Callable[[Any], Any]]

        if timeout is not None:
            request_opts += [set_timeout(timeout)]

        if tenant_key is not None:
            request_opts += [set_tenant_key(tenant_key)]

        if user_access_token is not None:
            request_opts += [set_user_access_token(user_access_token)]

        return CalendarEventAttendeeChatMemberListReqCall(self, request_opts=request_opts)



class CalendarCreateReqCall(object):
    def __init__(self, service, body, request_opts=None):
        # type: (CalendarService, Calendar, List[Any]) -> None

        self.service = service
        self.body = body

        if request_opts:
            self.request_opts = request_opts
        else:
            self.request_opts = []  # type: List[Any]

    def do(self):
        # type: () -> Response[CalendarCreateResult]
        root_service = self.service.service

        conf = root_service.conf
        req = Request('calendar/v4/calendars', 'POST', [ACCESS_TOKEN_TYPE_TENANT, ACCESS_TOKEN_TYPE_USER],
                      self.body, output_class=CalendarCreateResult, request_opts=self.request_opts)
        resp = req.do(conf)
        return resp


class CalendarEventDeleteReqCall(object):
    def __init__(self, service, request_opts=None):
        # type: (CalendarEventService, List[Any]) -> None

        self.service = service
        
        self.path_params = {}   # type: Dict[str, Any]

        if request_opts:
            self.request_opts = request_opts
        else:
            self.request_opts = []  # type: List[Any]

    def set_calendar_id(self, calendar_id):
        # type: (str) -> CalendarEventDeleteReqCall
        self.path_params['calendar_id'] = calendar_id
        return self

    def set_event_id(self, event_id):
        # type: (str) -> CalendarEventDeleteReqCall
        self.path_params['event_id'] = event_id
        return self

    def do(self):
        # type: () -> Response[None]
        root_service = self.service.service

        conf = root_service.conf
        self.request_opts += [set_path_params(self.path_params)]
        req = Request('calendar/v4/calendars/:calendar_id/events/:event_id', 'DELETE', [ACCESS_TOKEN_TYPE_TENANT, ACCESS_TOKEN_TYPE_USER],
                      None, request_opts=self.request_opts)
        resp = req.do(conf)
        return resp


class CalendarEventGetReqCall(object):
    def __init__(self, service, request_opts=None):
        # type: (CalendarEventService, List[Any]) -> None

        self.service = service
        
        self.path_params = {}   # type: Dict[str, Any]

        if request_opts:
            self.request_opts = request_opts
        else:
            self.request_opts = []  # type: List[Any]

    def set_calendar_id(self, calendar_id):
        # type: (str) -> CalendarEventGetReqCall
        self.path_params['calendar_id'] = calendar_id
        return self

    def set_event_id(self, event_id):
        # type: (str) -> CalendarEventGetReqCall
        self.path_params['event_id'] = event_id
        return self

    def do(self):
        # type: () -> Response[CalendarEventGetResult]
        root_service = self.service.service

        conf = root_service.conf
        self.request_opts += [set_path_params(self.path_params)]
        req = Request('calendar/v4/calendars/:calendar_id/events/:event_id', 'GET', [ACCESS_TOKEN_TYPE_TENANT, ACCESS_TOKEN_TYPE_USER],
                      None, output_class=CalendarEventGetResult, request_opts=self.request_opts)
        resp = req.do(conf)
        return resp


class CalendarPatchReqCall(object):
    def __init__(self, service, body, request_opts=None):
        # type: (CalendarService, Calendar, List[Any]) -> None

        self.service = service
        self.body = body
        self.path_params = {}   # type: Dict[str, Any]

        if request_opts:
            self.request_opts = request_opts
        else:
            self.request_opts = []  # type: List[Any]

    def set_calendar_id(self, calendar_id):
        # type: (str) -> CalendarPatchReqCall
        self.path_params['calendar_id'] = calendar_id
        return self

    def do(self):
        # type: () -> Response[CalendarPatchResult]
        root_service = self.service.service

        conf = root_service.conf
        self.request_opts += [set_path_params(self.path_params)]
        req = Request('calendar/v4/calendars/:calendar_id', 'PATCH', [ACCESS_TOKEN_TYPE_TENANT, ACCESS_TOKEN_TYPE_USER],
                      self.body, output_class=CalendarPatchResult, request_opts=self.request_opts)
        resp = req.do(conf)
        return resp


class CalendarDeleteReqCall(object):
    def __init__(self, service, request_opts=None):
        # type: (CalendarService, List[Any]) -> None

        self.service = service
        
        self.path_params = {}   # type: Dict[str, Any]

        if request_opts:
            self.request_opts = request_opts
        else:
            self.request_opts = []  # type: List[Any]

    def set_calendar_id(self, calendar_id):
        # type: (str) -> CalendarDeleteReqCall
        self.path_params['calendar_id'] = calendar_id
        return self

    def do(self):
        # type: () -> Response[None]
        root_service = self.service.service

        conf = root_service.conf
        self.request_opts += [set_path_params(self.path_params)]
        req = Request('calendar/v4/calendars/:calendar_id', 'DELETE', [ACCESS_TOKEN_TYPE_TENANT, ACCESS_TOKEN_TYPE_USER],
                      None, request_opts=self.request_opts)
        resp = req.do(conf)
        return resp


class CalendarAclListReqCall(object):
    def __init__(self, service, request_opts=None):
        # type: (CalendarAclService, List[Any]) -> None

        self.service = service
        
        self.path_params = {}   # type: Dict[str, Any]
        self.query_params = {}  # type: Dict[str, Any]

        if request_opts:
            self.request_opts = request_opts
        else:
            self.request_opts = []  # type: List[Any]

    def set_calendar_id(self, calendar_id):
        # type: (str) -> CalendarAclListReqCall
        self.path_params['calendar_id'] = calendar_id
        return self

    def set_user_id_type(self, user_id_type):
        # type: (str) -> CalendarAclListReqCall
        self.query_params['user_id_type'] = user_id_type
        return self

    def set_page_token(self, page_token):
        # type: (str) -> CalendarAclListReqCall
        self.query_params['page_token'] = page_token
        return self

    def set_page_size(self, page_size):
        # type: (int) -> CalendarAclListReqCall
        self.query_params['page_size'] = page_size
        return self

    def do(self):
        # type: () -> Response[CalendarAclListResult]
        root_service = self.service.service

        conf = root_service.conf
        self.request_opts += [set_path_params(self.path_params)]
        self.request_opts += [set_query_params(self.query_params)]
        req = Request('calendar/v4/calendars/:calendar_id/acls', 'GET', [ACCESS_TOKEN_TYPE_TENANT, ACCESS_TOKEN_TYPE_USER],
                      None, output_class=CalendarAclListResult, request_opts=self.request_opts)
        resp = req.do(conf)
        return resp


class CalendarAclDeleteReqCall(object):
    def __init__(self, service, request_opts=None):
        # type: (CalendarAclService, List[Any]) -> None

        self.service = service
        
        self.path_params = {}   # type: Dict[str, Any]

        if request_opts:
            self.request_opts = request_opts
        else:
            self.request_opts = []  # type: List[Any]

    def set_calendar_id(self, calendar_id):
        # type: (str) -> CalendarAclDeleteReqCall
        self.path_params['calendar_id'] = calendar_id
        return self

    def set_acl_id(self, acl_id):
        # type: (str) -> CalendarAclDeleteReqCall
        self.path_params['acl_id'] = acl_id
        return self

    def do(self):
        # type: () -> Response[None]
        root_service = self.service.service

        conf = root_service.conf
        self.request_opts += [set_path_params(self.path_params)]
        req = Request('calendar/v4/calendars/:calendar_id/acls/:acl_id', 'DELETE', [ACCESS_TOKEN_TYPE_TENANT, ACCESS_TOKEN_TYPE_USER],
                      None, request_opts=self.request_opts)
        resp = req.do(conf)
        return resp


class CalendarAclCreateReqCall(object):
    def __init__(self, service, body, request_opts=None):
        # type: (CalendarAclService, CalendarAcl, List[Any]) -> None

        self.service = service
        self.body = body
        self.path_params = {}   # type: Dict[str, Any]
        self.query_params = {}  # type: Dict[str, Any]

        if request_opts:
            self.request_opts = request_opts
        else:
            self.request_opts = []  # type: List[Any]

    def set_calendar_id(self, calendar_id):
        # type: (str) -> CalendarAclCreateReqCall
        self.path_params['calendar_id'] = calendar_id
        return self

    def set_user_id_type(self, user_id_type):
        # type: (str) -> CalendarAclCreateReqCall
        self.query_params['user_id_type'] = user_id_type
        return self

    def do(self):
        # type: () -> Response[CalendarAcl]
        root_service = self.service.service

        conf = root_service.conf
        self.request_opts += [set_path_params(self.path_params)]
        self.request_opts += [set_query_params(self.query_params)]
        req = Request('calendar/v4/calendars/:calendar_id/acls', 'POST', [ACCESS_TOKEN_TYPE_TENANT, ACCESS_TOKEN_TYPE_USER],
                      self.body, output_class=CalendarAcl, request_opts=self.request_opts)
        resp = req.do(conf)
        return resp


class CalendarEventCreateReqCall(object):
    def __init__(self, service, body, request_opts=None):
        # type: (CalendarEventService, CalendarEvent, List[Any]) -> None

        self.service = service
        self.body = body
        self.path_params = {}   # type: Dict[str, Any]

        if request_opts:
            self.request_opts = request_opts
        else:
            self.request_opts = []  # type: List[Any]

    def set_calendar_id(self, calendar_id):
        # type: (str) -> CalendarEventCreateReqCall
        self.path_params['calendar_id'] = calendar_id
        return self

    def do(self):
        # type: () -> Response[CalendarEventCreateResult]
        root_service = self.service.service

        conf = root_service.conf
        self.request_opts += [set_path_params(self.path_params)]
        req = Request('calendar/v4/calendars/:calendar_id/events', 'POST', [ACCESS_TOKEN_TYPE_TENANT, ACCESS_TOKEN_TYPE_USER],
                      self.body, output_class=CalendarEventCreateResult, request_opts=self.request_opts)
        resp = req.do(conf)
        return resp


class CalendarListReqCall(object):
    def __init__(self, service, request_opts=None):
        # type: (CalendarService, List[Any]) -> None

        self.service = service
        
        self.query_params = {}  # type: Dict[str, Any]

        if request_opts:
            self.request_opts = request_opts
        else:
            self.request_opts = []  # type: List[Any]

    def set_page_size(self, page_size):
        # type: (int) -> CalendarListReqCall
        self.query_params['page_size'] = page_size
        return self

    def set_page_token(self, page_token):
        # type: (str) -> CalendarListReqCall
        self.query_params['page_token'] = page_token
        return self

    def set_sync_token(self, sync_token):
        # type: (str) -> CalendarListReqCall
        self.query_params['sync_token'] = sync_token
        return self

    def do(self):
        # type: () -> Response[CalendarListResult]
        root_service = self.service.service

        conf = root_service.conf
        self.request_opts += [set_query_params(self.query_params)]
        req = Request('calendar/v4/calendars', 'GET', [ACCESS_TOKEN_TYPE_TENANT, ACCESS_TOKEN_TYPE_USER],
                      None, output_class=CalendarListResult, request_opts=self.request_opts)
        resp = req.do(conf)
        return resp


class CalendarEventAttendeeListReqCall(object):
    def __init__(self, service, request_opts=None):
        # type: (CalendarEventAttendeeService, List[Any]) -> None

        self.service = service
        
        self.path_params = {}   # type: Dict[str, Any]
        self.query_params = {}  # type: Dict[str, Any]

        if request_opts:
            self.request_opts = request_opts
        else:
            self.request_opts = []  # type: List[Any]

    def set_calendar_id(self, calendar_id):
        # type: (str) -> CalendarEventAttendeeListReqCall
        self.path_params['calendar_id'] = calendar_id
        return self

    def set_event_id(self, event_id):
        # type: (str) -> CalendarEventAttendeeListReqCall
        self.path_params['event_id'] = event_id
        return self

    def set_user_id_type(self, user_id_type):
        # type: (str) -> CalendarEventAttendeeListReqCall
        self.query_params['user_id_type'] = user_id_type
        return self

    def set_page_token(self, page_token):
        # type: (str) -> CalendarEventAttendeeListReqCall
        self.query_params['page_token'] = page_token
        return self

    def set_page_size(self, page_size):
        # type: (int) -> CalendarEventAttendeeListReqCall
        self.query_params['page_size'] = page_size
        return self

    def do(self):
        # type: () -> Response[CalendarEventAttendeeListResult]
        root_service = self.service.service

        conf = root_service.conf
        self.request_opts += [set_path_params(self.path_params)]
        self.request_opts += [set_query_params(self.query_params)]
        req = Request('calendar/v4/calendars/:calendar_id/events/:event_id/attendees', 'GET', [ACCESS_TOKEN_TYPE_TENANT, ACCESS_TOKEN_TYPE_USER],
                      None, output_class=CalendarEventAttendeeListResult, request_opts=self.request_opts)
        resp = req.do(conf)
        return resp


class CalendarEventAttendeeBatchDeleteReqCall(object):
    def __init__(self, service, body, request_opts=None):
        # type: (CalendarEventAttendeeService, CalendarEventAttendeeBatchDeleteReqBody, List[Any]) -> None

        self.service = service
        self.body = body
        self.path_params = {}   # type: Dict[str, Any]

        if request_opts:
            self.request_opts = request_opts
        else:
            self.request_opts = []  # type: List[Any]

    def set_calendar_id(self, calendar_id):
        # type: (str) -> CalendarEventAttendeeBatchDeleteReqCall
        self.path_params['calendar_id'] = calendar_id
        return self

    def set_event_id(self, event_id):
        # type: (str) -> CalendarEventAttendeeBatchDeleteReqCall
        self.path_params['event_id'] = event_id
        return self

    def do(self):
        # type: () -> Response[None]
        root_service = self.service.service

        conf = root_service.conf
        self.request_opts += [set_path_params(self.path_params)]
        req = Request('calendar/v4/calendars/:calendar_id/events/:event_id/attendees/batch_delete', 'POST', [ACCESS_TOKEN_TYPE_TENANT, ACCESS_TOKEN_TYPE_USER],
                      self.body, request_opts=self.request_opts)
        resp = req.do(conf)
        return resp


class CalendarEventAttendeeCreateReqCall(object):
    def __init__(self, service, body, request_opts=None):
        # type: (CalendarEventAttendeeService, CalendarEventAttendeeCreateReqBody, List[Any]) -> None

        self.service = service
        self.body = body
        self.path_params = {}   # type: Dict[str, Any]
        self.query_params = {}  # type: Dict[str, Any]

        if request_opts:
            self.request_opts = request_opts
        else:
            self.request_opts = []  # type: List[Any]

    def set_calendar_id(self, calendar_id):
        # type: (str) -> CalendarEventAttendeeCreateReqCall
        self.path_params['calendar_id'] = calendar_id
        return self

    def set_event_id(self, event_id):
        # type: (str) -> CalendarEventAttendeeCreateReqCall
        self.path_params['event_id'] = event_id
        return self

    def set_user_id_type(self, user_id_type):
        # type: (str) -> CalendarEventAttendeeCreateReqCall
        self.query_params['user_id_type'] = user_id_type
        return self

    def do(self):
        # type: () -> Response[CalendarEventAttendeeCreateResult]
        root_service = self.service.service

        conf = root_service.conf
        self.request_opts += [set_path_params(self.path_params)]
        self.request_opts += [set_query_params(self.query_params)]
        req = Request('calendar/v4/calendars/:calendar_id/events/:event_id/attendees', 'POST', [ACCESS_TOKEN_TYPE_TENANT, ACCESS_TOKEN_TYPE_USER],
                      self.body, output_class=CalendarEventAttendeeCreateResult, request_opts=self.request_opts)
        resp = req.do(conf)
        return resp


class CalendarGetReqCall(object):
    def __init__(self, service, request_opts=None):
        # type: (CalendarService, List[Any]) -> None

        self.service = service
        
        self.path_params = {}   # type: Dict[str, Any]

        if request_opts:
            self.request_opts = request_opts
        else:
            self.request_opts = []  # type: List[Any]

    def set_calendar_id(self, calendar_id):
        # type: (str) -> CalendarGetReqCall
        self.path_params['calendar_id'] = calendar_id
        return self

    def do(self):
        # type: () -> Response[Calendar]
        root_service = self.service.service

        conf = root_service.conf
        self.request_opts += [set_path_params(self.path_params)]
        req = Request('calendar/v4/calendars/:calendar_id', 'GET', [ACCESS_TOKEN_TYPE_TENANT, ACCESS_TOKEN_TYPE_USER],
                      None, output_class=Calendar, request_opts=self.request_opts)
        resp = req.do(conf)
        return resp


class CalendarEventListReqCall(object):
    def __init__(self, service, request_opts=None):
        # type: (CalendarEventService, List[Any]) -> None

        self.service = service
        
        self.path_params = {}   # type: Dict[str, Any]
        self.query_params = {}  # type: Dict[str, Any]

        if request_opts:
            self.request_opts = request_opts
        else:
            self.request_opts = []  # type: List[Any]

    def set_calendar_id(self, calendar_id):
        # type: (str) -> CalendarEventListReqCall
        self.path_params['calendar_id'] = calendar_id
        return self

    def set_page_size(self, page_size):
        # type: (int) -> CalendarEventListReqCall
        self.query_params['page_size'] = page_size
        return self

    def set_page_token(self, page_token):
        # type: (str) -> CalendarEventListReqCall
        self.query_params['page_token'] = page_token
        return self

    def set_sync_token(self, sync_token):
        # type: (str) -> CalendarEventListReqCall
        self.query_params['sync_token'] = sync_token
        return self

    def do(self):
        # type: () -> Response[CalendarEventListResult]
        root_service = self.service.service

        conf = root_service.conf
        self.request_opts += [set_path_params(self.path_params)]
        self.request_opts += [set_query_params(self.query_params)]
        req = Request('calendar/v4/calendars/:calendar_id/events', 'GET', [ACCESS_TOKEN_TYPE_TENANT, ACCESS_TOKEN_TYPE_USER],
                      None, output_class=CalendarEventListResult, request_opts=self.request_opts)
        resp = req.do(conf)
        return resp


class CalendarSearchReqCall(object):
    def __init__(self, service, body, request_opts=None):
        # type: (CalendarService, CalendarSearchReqBody, List[Any]) -> None

        self.service = service
        self.body = body
        self.query_params = {}  # type: Dict[str, Any]

        if request_opts:
            self.request_opts = request_opts
        else:
            self.request_opts = []  # type: List[Any]

    def set_page_token(self, page_token):
        # type: (str) -> CalendarSearchReqCall
        self.query_params['page_token'] = page_token
        return self

    def set_page_size(self, page_size):
        # type: (int) -> CalendarSearchReqCall
        self.query_params['page_size'] = page_size
        return self

    def do(self):
        # type: () -> Response[CalendarSearchResult]
        root_service = self.service.service

        conf = root_service.conf
        self.request_opts += [set_query_params(self.query_params)]
        req = Request('calendar/v4/calendars/search', 'POST', [ACCESS_TOKEN_TYPE_TENANT],
                      self.body, output_class=CalendarSearchResult, request_opts=self.request_opts)
        resp = req.do(conf)
        return resp


class FreebusyListReqCall(object):
    def __init__(self, service, body, request_opts=None):
        # type: (FreebusyService, FreebusyListReqBody, List[Any]) -> None

        self.service = service
        self.body = body
        self.query_params = {}  # type: Dict[str, Any]

        if request_opts:
            self.request_opts = request_opts
        else:
            self.request_opts = []  # type: List[Any]

    def set_user_id_type(self, user_id_type):
        # type: (str) -> FreebusyListReqCall
        self.query_params['user_id_type'] = user_id_type
        return self

    def do(self):
        # type: () -> Response[FreebusyListResult]
        root_service = self.service.service

        conf = root_service.conf
        self.request_opts += [set_query_params(self.query_params)]
        req = Request('calendar/v4/freebusy/list', 'POST', [ACCESS_TOKEN_TYPE_TENANT],
                      self.body, output_class=FreebusyListResult, request_opts=self.request_opts)
        resp = req.do(conf)
        return resp


class CalendarEventPatchReqCall(object):
    def __init__(self, service, body, request_opts=None):
        # type: (CalendarEventService, CalendarEvent, List[Any]) -> None

        self.service = service
        self.body = body
        self.path_params = {}   # type: Dict[str, Any]

        if request_opts:
            self.request_opts = request_opts
        else:
            self.request_opts = []  # type: List[Any]

    def set_calendar_id(self, calendar_id):
        # type: (str) -> CalendarEventPatchReqCall
        self.path_params['calendar_id'] = calendar_id
        return self

    def set_event_id(self, event_id):
        # type: (str) -> CalendarEventPatchReqCall
        self.path_params['event_id'] = event_id
        return self

    def do(self):
        # type: () -> Response[CalendarEventPatchResult]
        root_service = self.service.service

        conf = root_service.conf
        self.request_opts += [set_path_params(self.path_params)]
        req = Request('calendar/v4/calendars/:calendar_id/events/:event_id', 'PATCH', [ACCESS_TOKEN_TYPE_TENANT, ACCESS_TOKEN_TYPE_USER],
                      self.body, output_class=CalendarEventPatchResult, request_opts=self.request_opts)
        resp = req.do(conf)
        return resp


class TimeoffEventDeleteReqCall(object):
    def __init__(self, service, request_opts=None):
        # type: (TimeoffEventService, List[Any]) -> None

        self.service = service
        
        self.path_params = {}   # type: Dict[str, Any]

        if request_opts:
            self.request_opts = request_opts
        else:
            self.request_opts = []  # type: List[Any]

    def set_timeoff_event_id(self, timeoff_event_id):
        # type: (str) -> TimeoffEventDeleteReqCall
        self.path_params['timeoff_event_id'] = timeoff_event_id
        return self

    def do(self):
        # type: () -> Response[None]
        root_service = self.service.service

        conf = root_service.conf
        self.request_opts += [set_path_params(self.path_params)]
        req = Request('calendar/v4/timeoff_events/:timeoff_event_id', 'DELETE', [ACCESS_TOKEN_TYPE_TENANT],
                      None, request_opts=self.request_opts)
        resp = req.do(conf)
        return resp


class TimeoffEventCreateReqCall(object):
    def __init__(self, service, body, request_opts=None):
        # type: (TimeoffEventService, TimeoffEvent, List[Any]) -> None

        self.service = service
        self.body = body
        self.query_params = {}  # type: Dict[str, Any]

        if request_opts:
            self.request_opts = request_opts
        else:
            self.request_opts = []  # type: List[Any]

    def set_user_id_type(self, user_id_type):
        # type: (str) -> TimeoffEventCreateReqCall
        self.query_params['user_id_type'] = user_id_type
        return self

    def do(self):
        # type: () -> Response[TimeoffEvent]
        root_service = self.service.service

        conf = root_service.conf
        self.request_opts += [set_query_params(self.query_params)]
        req = Request('calendar/v4/timeoff_events', 'POST', [ACCESS_TOKEN_TYPE_TENANT],
                      self.body, output_class=TimeoffEvent, request_opts=self.request_opts)
        resp = req.do(conf)
        return resp


class CalendarUnsubscribeReqCall(object):
    def __init__(self, service, request_opts=None):
        # type: (CalendarService, List[Any]) -> None

        self.service = service
        
        self.path_params = {}   # type: Dict[str, Any]

        if request_opts:
            self.request_opts = request_opts
        else:
            self.request_opts = []  # type: List[Any]

    def set_calendar_id(self, calendar_id):
        # type: (str) -> CalendarUnsubscribeReqCall
        self.path_params['calendar_id'] = calendar_id
        return self

    def do(self):
        # type: () -> Response[None]
        root_service = self.service.service

        conf = root_service.conf
        self.request_opts += [set_path_params(self.path_params)]
        req = Request('calendar/v4/calendars/:calendar_id/unsubscribe', 'POST', [ACCESS_TOKEN_TYPE_USER, ACCESS_TOKEN_TYPE_TENANT],
                      None, request_opts=self.request_opts)
        resp = req.do(conf)
        return resp


class CalendarEventSearchReqCall(object):
    def __init__(self, service, body, request_opts=None):
        # type: (CalendarEventService, CalendarEventSearchReqBody, List[Any]) -> None

        self.service = service
        self.body = body
        self.path_params = {}   # type: Dict[str, Any]
        self.query_params = {}  # type: Dict[str, Any]

        if request_opts:
            self.request_opts = request_opts
        else:
            self.request_opts = []  # type: List[Any]

    def set_calendar_id(self, calendar_id):
        # type: (str) -> CalendarEventSearchReqCall
        self.path_params['calendar_id'] = calendar_id
        return self

    def set_user_id_type(self, user_id_type):
        # type: (str) -> CalendarEventSearchReqCall
        self.query_params['user_id_type'] = user_id_type
        return self

    def set_page_token(self, page_token):
        # type: (str) -> CalendarEventSearchReqCall
        self.query_params['page_token'] = page_token
        return self

    def set_page_size(self, page_size):
        # type: (int) -> CalendarEventSearchReqCall
        self.query_params['page_size'] = page_size
        return self

    def do(self):
        # type: () -> Response[CalendarEventSearchResult]
        root_service = self.service.service

        conf = root_service.conf
        self.request_opts += [set_path_params(self.path_params)]
        self.request_opts += [set_query_params(self.query_params)]
        req = Request('calendar/v4/calendars/:calendar_id/events/search', 'POST', [ACCESS_TOKEN_TYPE_USER],
                      self.body, output_class=CalendarEventSearchResult, request_opts=self.request_opts)
        resp = req.do(conf)
        return resp


class CalendarSubscribeReqCall(object):
    def __init__(self, service, request_opts=None):
        # type: (CalendarService, List[Any]) -> None

        self.service = service
        
        self.path_params = {}   # type: Dict[str, Any]

        if request_opts:
            self.request_opts = request_opts
        else:
            self.request_opts = []  # type: List[Any]

    def set_calendar_id(self, calendar_id):
        # type: (str) -> CalendarSubscribeReqCall
        self.path_params['calendar_id'] = calendar_id
        return self

    def do(self):
        # type: () -> Response[CalendarSubscribeResult]
        root_service = self.service.service

        conf = root_service.conf
        self.request_opts += [set_path_params(self.path_params)]
        req = Request('calendar/v4/calendars/:calendar_id/subscribe', 'POST', [ACCESS_TOKEN_TYPE_USER, ACCESS_TOKEN_TYPE_TENANT],
                      None, output_class=CalendarSubscribeResult, request_opts=self.request_opts)
        resp = req.do(conf)
        return resp


class SettingGenerateCaldavConfReqCall(object):
    def __init__(self, service, body, request_opts=None):
        # type: (SettingService, SettingGenerateCaldavConfReqBody, List[Any]) -> None

        self.service = service
        self.body = body

        if request_opts:
            self.request_opts = request_opts
        else:
            self.request_opts = []  # type: List[Any]

    def do(self):
        # type: () -> Response[SettingGenerateCaldavConfResult]
        root_service = self.service.service

        conf = root_service.conf
        req = Request('calendar/v4/settings/generate_caldav_conf', 'POST', [ACCESS_TOKEN_TYPE_USER],
                      self.body, output_class=SettingGenerateCaldavConfResult, request_opts=self.request_opts)
        resp = req.do(conf)
        return resp


class CalendarEventSubscriptionReqCall(object):
    def __init__(self, service, request_opts=None):
        # type: (CalendarEventService, List[Any]) -> None

        self.service = service
        
        self.path_params = {}   # type: Dict[str, Any]

        if request_opts:
            self.request_opts = request_opts
        else:
            self.request_opts = []  # type: List[Any]

    def set_calendar_id(self, calendar_id):
        # type: (str) -> CalendarEventSubscriptionReqCall
        self.path_params['calendar_id'] = calendar_id
        return self

    def do(self):
        # type: () -> Response[None]
        root_service = self.service.service

        conf = root_service.conf
        self.request_opts += [set_path_params(self.path_params)]
        req = Request('calendar/v4/calendars/:calendar_id/events/subscription', 'POST', [ACCESS_TOKEN_TYPE_USER],
                      None, request_opts=self.request_opts)
        resp = req.do(conf)
        return resp


class CalendarSubscriptionReqCall(object):
    def __init__(self, service, request_opts=None):
        # type: (CalendarService, List[Any]) -> None

        self.service = service
        

        if request_opts:
            self.request_opts = request_opts
        else:
            self.request_opts = []  # type: List[Any]

    def do(self):
        # type: () -> Response[None]
        root_service = self.service.service

        conf = root_service.conf
        req = Request('calendar/v4/calendars/subscription', 'POST', [ACCESS_TOKEN_TYPE_USER],
                      None, request_opts=self.request_opts)
        resp = req.do(conf)
        return resp


class CalendarAclSubscriptionReqCall(object):
    def __init__(self, service, request_opts=None):
        # type: (CalendarAclService, List[Any]) -> None

        self.service = service
        
        self.path_params = {}   # type: Dict[str, Any]

        if request_opts:
            self.request_opts = request_opts
        else:
            self.request_opts = []  # type: List[Any]

    def set_calendar_id(self, calendar_id):
        # type: (str) -> CalendarAclSubscriptionReqCall
        self.path_params['calendar_id'] = calendar_id
        return self

    def do(self):
        # type: () -> Response[None]
        root_service = self.service.service

        conf = root_service.conf
        self.request_opts += [set_path_params(self.path_params)]
        req = Request('calendar/v4/calendars/:calendar_id/acls/subscription', 'POST', [ACCESS_TOKEN_TYPE_USER],
                      None, request_opts=self.request_opts)
        resp = req.do(conf)
        return resp


class CalendarEventAttendeeChatMemberListReqCall(object):
    def __init__(self, service, request_opts=None):
        # type: (CalendarEventAttendeeChatMemberService, List[Any]) -> None

        self.service = service
        
        self.path_params = {}   # type: Dict[str, Any]
        self.query_params = {}  # type: Dict[str, Any]

        if request_opts:
            self.request_opts = request_opts
        else:
            self.request_opts = []  # type: List[Any]

    def set_calendar_id(self, calendar_id):
        # type: (str) -> CalendarEventAttendeeChatMemberListReqCall
        self.path_params['calendar_id'] = calendar_id
        return self

    def set_event_id(self, event_id):
        # type: (str) -> CalendarEventAttendeeChatMemberListReqCall
        self.path_params['event_id'] = event_id
        return self

    def set_attendee_id(self, attendee_id):
        # type: (str) -> CalendarEventAttendeeChatMemberListReqCall
        self.path_params['attendee_id'] = attendee_id
        return self

    def set_page_token(self, page_token):
        # type: (str) -> CalendarEventAttendeeChatMemberListReqCall
        self.query_params['page_token'] = page_token
        return self

    def set_page_size(self, page_size):
        # type: (int) -> CalendarEventAttendeeChatMemberListReqCall
        self.query_params['page_size'] = page_size
        return self

    def do(self):
        # type: () -> Response[CalendarEventAttendeeChatMemberListResult]
        root_service = self.service.service

        conf = root_service.conf
        self.request_opts += [set_path_params(self.path_params)]
        self.request_opts += [set_query_params(self.query_params)]
        req = Request('calendar/v4/calendars/:calendar_id/events/:event_id/attendees/:attendee_id/chat_members', 'GET', [ACCESS_TOKEN_TYPE_TENANT, ACCESS_TOKEN_TYPE_USER],
                      None, output_class=CalendarEventAttendeeChatMemberListResult, request_opts=self.request_opts)
        resp = req.do(conf)
        return resp




# -*- coding: UTF-8 -*-
# Code generated by lark suite oapi sdk gen

from typing import Callable

from ....config import Config
from ....context import Context
from ....event.event import set_event_callback

from .model import *


class RoomCreatedEventHandler(object):
    def __init__(self, callback):
        # type: (Callable[[Context, Config, RoomCreatedEvent], Any]) -> None
        self.handler = callback

    def handle(self, ctx, conf, event):  # type: (Context, Config, RoomCreatedEvent) -> Any
        return self.handler(ctx, conf, event)

    @staticmethod
    def set_callback(conf, callback):
        # type: (Config, Callable[[Context, Config, RoomCreatedEvent], Any]) -> None
        handler = RoomCreatedEventHandler(callback)
        set_event_callback(conf, "meeting_room.room.created_v1",
                          handler.handle, clazz=RoomCreatedEvent)


class RoomDeletedEventHandler(object):
    def __init__(self, callback):
        # type: (Callable[[Context, Config, RoomDeletedEvent], Any]) -> None
        self.handler = callback

    def handle(self, ctx, conf, event):  # type: (Context, Config, RoomDeletedEvent) -> Any
        return self.handler(ctx, conf, event)

    @staticmethod
    def set_callback(conf, callback):
        # type: (Config, Callable[[Context, Config, RoomDeletedEvent], Any]) -> None
        handler = RoomDeletedEventHandler(callback)
        set_event_callback(conf, "meeting_room.room.deleted_v1",
                          handler.handle, clazz=RoomDeletedEvent)


class RoomStatusChangedEventHandler(object):
    def __init__(self, callback):
        # type: (Callable[[Context, Config, RoomStatusChangedEvent], Any]) -> None
        self.handler = callback

    def handle(self, ctx, conf, event):  # type: (Context, Config, RoomStatusChangedEvent) -> Any
        return self.handler(ctx, conf, event)

    @staticmethod
    def set_callback(conf, callback):
        # type: (Config, Callable[[Context, Config, RoomStatusChangedEvent], Any]) -> None
        handler = RoomStatusChangedEventHandler(callback)
        set_event_callback(conf, "meeting_room.room.status_changed_v1",
                          handler.handle, clazz=RoomStatusChangedEvent)


class RoomUpdatedEventHandler(object):
    def __init__(self, callback):
        # type: (Callable[[Context, Config, RoomUpdatedEvent], Any]) -> None
        self.handler = callback

    def handle(self, ctx, conf, event):  # type: (Context, Config, RoomUpdatedEvent) -> Any
        return self.handler(ctx, conf, event)

    @staticmethod
    def set_callback(conf, callback):
        # type: (Config, Callable[[Context, Config, RoomUpdatedEvent], Any]) -> None
        handler = RoomUpdatedEventHandler(callback)
        set_event_callback(conf, "meeting_room.room.updated_v1",
                          handler.handle, clazz=RoomUpdatedEvent)


class MeetingRoomStatusChangedEventHandler(object):
    def __init__(self, callback):
        # type: (Callable[[Context, Config, MeetingRoomStatusChangedEvent], Any]) -> None
        self.handler = callback

    def handle(self, ctx, conf, event):  # type: (Context, Config, MeetingRoomStatusChangedEvent) -> Any
        return self.handler(ctx, conf, event)

    @staticmethod
    def set_callback(conf, callback):
        # type: (Config, Callable[[Context, Config, MeetingRoomStatusChangedEvent], Any]) -> None
        handler = MeetingRoomStatusChangedEventHandler(callback)
        set_event_callback(conf, "meeting_room.meeting_room.status_changed_v1",
                          handler.handle, clazz=MeetingRoomStatusChangedEvent)


class MeetingRoomCreatedEventHandler(object):
    def __init__(self, callback):
        # type: (Callable[[Context, Config, MeetingRoomCreatedEvent], Any]) -> None
        self.handler = callback

    def handle(self, ctx, conf, event):  # type: (Context, Config, MeetingRoomCreatedEvent) -> Any
        return self.handler(ctx, conf, event)

    @staticmethod
    def set_callback(conf, callback):
        # type: (Config, Callable[[Context, Config, MeetingRoomCreatedEvent], Any]) -> None
        handler = MeetingRoomCreatedEventHandler(callback)
        set_event_callback(conf, "meeting_room.meeting_room.created_v1",
                          handler.handle, clazz=MeetingRoomCreatedEvent)


class MeetingRoomDeletedEventHandler(object):
    def __init__(self, callback):
        # type: (Callable[[Context, Config, MeetingRoomDeletedEvent], Any]) -> None
        self.handler = callback

    def handle(self, ctx, conf, event):  # type: (Context, Config, MeetingRoomDeletedEvent) -> Any
        return self.handler(ctx, conf, event)

    @staticmethod
    def set_callback(conf, callback):
        # type: (Config, Callable[[Context, Config, MeetingRoomDeletedEvent], Any]) -> None
        handler = MeetingRoomDeletedEventHandler(callback)
        set_event_callback(conf, "meeting_room.meeting_room.deleted_v1",
                          handler.handle, clazz=MeetingRoomDeletedEvent)


class MeetingRoomUpdatedEventHandler(object):
    def __init__(self, callback):
        # type: (Callable[[Context, Config, MeetingRoomUpdatedEvent], Any]) -> None
        self.handler = callback

    def handle(self, ctx, conf, event):  # type: (Context, Config, MeetingRoomUpdatedEvent) -> Any
        return self.handler(ctx, conf, event)

    @staticmethod
    def set_callback(conf, callback):
        # type: (Config, Callable[[Context, Config, MeetingRoomUpdatedEvent], Any]) -> None
        handler = MeetingRoomUpdatedEventHandler(callback)
        set_event_callback(conf, "meeting_room.meeting_room.updated_v1",
                          handler.handle, clazz=MeetingRoomUpdatedEvent)



# -*- coding: UTF-8 -*-
# Code generated by lark suite oapi sdk gen

from typing import List, Dict, Any
from ....utils.dt import to_json_decorator
from ....event.model.event import *
import attr




@to_json_decorator
@attr.s
class OrganizerInfo(object):
    name = attr.ib(type=str, default=None, metadata={'json': 'name'})
    open_id = attr.ib(type=str, default=None, metadata={'json': 'open_id'})


@to_json_decorator
@attr.s
class RoomFreeBusy(object):
    start_time = attr.ib(type=str, default=None, metadata={'json': 'start_time'})
    end_time = attr.ib(type=str, default=None, metadata={'json': 'end_time'})
    uid = attr.ib(type=str, default=None, metadata={'json': 'uid'})
    original_time = attr.ib(type=int, default=None, metadata={'json': 'original_time'})
    organizer_info = attr.ib(type=OrganizerInfo, default=None, metadata={'json': 'organizer_info'})


@to_json_decorator
@attr.s
class Building(object):
    building_id = attr.ib(type=str, default=None, metadata={'json': 'building_id'})
    description = attr.ib(type=str, default=None, metadata={'json': 'description'})
    floors = attr.ib(type=List[str], default=None, metadata={'json': 'floors'})
    name = attr.ib(type=str, default=None, metadata={'json': 'name'})


@to_json_decorator
@attr.s
class BuildingId(object):
    building_id = attr.ib(type=str, default=None, metadata={'json': 'building_id'})
    custom_building_id = attr.ib(type=str, default=None, metadata={'json': 'custom_building_id'})


@to_json_decorator
@attr.s
class Country(object):
    country_id = attr.ib(type=str, default=None, metadata={'json': 'country_id'})
    name = attr.ib(type=str, default=None, metadata={'json': 'name'})


@to_json_decorator
@attr.s
class District(object):
    district_id = attr.ib(type=str, default=None, metadata={'json': 'district_id'})
    name = attr.ib(type=str, default=None, metadata={'json': 'name'})


@to_json_decorator
@attr.s
class ErrorEventUid(object):
    uid = attr.ib(type=str, default=None, metadata={'json': 'uid'})
    original_time = attr.ib(type=str, default=None, metadata={'json': 'original_time'})
    error_msg = attr.ib(type=str, default=None, metadata={'json': 'error_msg'})


@to_json_decorator
@attr.s
class EventInfo(object):
    uid = attr.ib(type=str, default=None, metadata={'json': 'uid'})
    original_time = attr.ib(type=str, default=None, metadata={'json': 'original_time'})
    summary = attr.ib(type=str, default=None, metadata={'json': 'summary'})


@to_json_decorator
@attr.s
class EventUid(object):
    uid = attr.ib(type=str, default=None, metadata={'json': 'uid'})
    original_time = attr.ib(type=int, default=None, metadata={'json': 'original_time'})


@to_json_decorator
@attr.s
class Freebusy(object):
    pass


@to_json_decorator
@attr.s
class Instance(object):
    pass


@to_json_decorator
@attr.s
class Room(object):
    room_id = attr.ib(type=str, default=None, metadata={'json': 'room_id'})
    building_id = attr.ib(type=str, default=None, metadata={'json': 'building_id'})
    building_name = attr.ib(type=str, default=None, metadata={'json': 'building_name'})
    capacity = attr.ib(type=int, default=None, metadata={'json': 'capacity'})
    description = attr.ib(type=str, default=None, metadata={'json': 'description'})
    display_id = attr.ib(type=str, default=None, metadata={'json': 'display_id'})
    floor_name = attr.ib(type=str, default=None, metadata={'json': 'floor_name'})
    is_disabled = attr.ib(type=bool, default=None, metadata={'json': 'is_disabled'})
    name = attr.ib(type=str, default=None, metadata={'json': 'name'})


@to_json_decorator
@attr.s
class RoomId(object):
    room_id = attr.ib(type=str, default=None, metadata={'json': 'room_id'})
    custom_room_id = attr.ib(type=str, default=None, metadata={'json': 'custom_room_id'})


@to_json_decorator
@attr.s
class Summary(object):
    pass


@to_json_decorator
@attr.s
class MeetingRoom(object):
    room_id = attr.ib(type=int, default=None, metadata={'json': 'room_id'})




@attr.s
class RoomBatchGetResult(object):
    rooms = attr.ib(type=List[Room], default=None, metadata={'json': 'rooms'})



@attr.s
class FreebusyBatchGetResult(object):
    time_min = attr.ib(type=str, default=None, metadata={'json': 'time_min'})
    time_max = attr.ib(type=str, default=None, metadata={'json': 'time_max'})
    free_busy = attr.ib(type=Dict[str, RoomFreeBusy], default=None, metadata={'json': 'free_busy'})


@to_json_decorator
@attr.s
class SummaryBatchGetReqBody(object):
    event_uids = attr.ib(type=List[EventUid], default=None, metadata={'json': 'EventUids'})


@attr.s
class SummaryBatchGetResult(object):
    event_infos = attr.ib(type=List[EventInfo], default=None, metadata={'json': 'EventInfos'})
    error_event_uids = attr.ib(type=List[ErrorEventUid], default=None, metadata={'json': 'ErrorEventUids'})



@attr.s
class BuildingBatchGetResult(object):
    buildings = attr.ib(type=List[Building], default=None, metadata={'json': 'buildings'})



@attr.s
class BuildingBatchGetIdResult(object):
    buildings = attr.ib(type=List[BuildingId], default=None, metadata={'json': 'buildings'})



@attr.s
class RoomBatchGetIdResult(object):
    rooms = attr.ib(type=List[RoomId], default=None, metadata={'json': 'rooms'})


@to_json_decorator
@attr.s
class BuildingCreateReqBody(object):
    name = attr.ib(type=str, default=None, metadata={'json': 'name'})
    floors = attr.ib(type=List[str], default=None, metadata={'json': 'floors'})
    country_id = attr.ib(type=str, default=None, metadata={'json': 'country_id'})
    district_id = attr.ib(type=str, default=None, metadata={'json': 'district_id'})
    custom_building_id = attr.ib(type=str, default=None, metadata={'json': 'custom_building_id'})


@attr.s
class BuildingCreateResult(object):
    building_id = attr.ib(type=str, default=None, metadata={'json': 'building_id'})


@to_json_decorator
@attr.s
class RoomCreateReqBody(object):
    building_id = attr.ib(type=str, default=None, metadata={'json': 'building_id'})
    floor = attr.ib(type=str, default=None, metadata={'json': 'floor'})
    name = attr.ib(type=str, default=None, metadata={'json': 'name'})
    capacity = attr.ib(type=int, default=None, metadata={'json': 'capacity'})
    is_disabled = attr.ib(type=bool, default=None, metadata={'json': 'is_disabled'})
    custom_room_id = attr.ib(type=str, default=None, metadata={'json': 'custom_room_id'})


@attr.s
class RoomCreateResult(object):
    room_id = attr.ib(type=str, default=None, metadata={'json': 'room_id'})







@attr.s
class CountryListResult(object):
    countries = attr.ib(type=List[Country], default=None, metadata={'json': 'countries'})



@attr.s
class RoomListResult(object):
    rooms = attr.ib(type=List[Room], default=None, metadata={'json': 'rooms'})



@attr.s
class DistrictListResult(object):
    districts = attr.ib(type=List[District], default=None, metadata={'json': 'districts'})



@attr.s
class BuildingListResult(object):
    buildings = attr.ib(type=List[Building], default=None, metadata={'json': 'buildings'})


@to_json_decorator
@attr.s
class InstanceReplyReqBody(object):
    room_id = attr.ib(type=str, default=None, metadata={'json': 'room_id'})
    uid = attr.ib(type=str, default=None, metadata={'json': 'uid'})
    original_time = attr.ib(type=int, default=None, metadata={'json': 'original_time'})
    status = attr.ib(type=str, default=None, metadata={'json': 'status'})



@to_json_decorator
@attr.s
class RoomUpdateReqBody(object):
    name = attr.ib(type=str, default=None, metadata={'json': 'name'})
    capacity = attr.ib(type=int, default=None, metadata={'json': 'capacity'})
    is_disabled = attr.ib(type=bool, default=None, metadata={'json': 'is_disabled'})
    custom_room_id = attr.ib(type=str, default=None, metadata={'json': 'custom_room_id'})



@to_json_decorator
@attr.s
class BuildingUpdateReqBody(object):
    name = attr.ib(type=str, default=None, metadata={'json': 'name'})
    floors = attr.ib(type=List[str], default=None, metadata={'json': 'floors'})
    country_id = attr.ib(type=str, default=None, metadata={'json': 'country_id'})
    district_id = attr.ib(type=str, default=None, metadata={'json': 'district_id'})
    custom_building_id = attr.ib(type=str, default=None, metadata={'json': 'custom_building_id'})



@attr.s
class RoomCreatedEventData(object):
    room_id = attr.ib(type=str, default=None, metadata={'json': 'room_id'})
    room_name = attr.ib(type=str, default=None, metadata={'json': 'room_name'})


@attr.s
class RoomCreatedEvent(BaseEventV2):
    event = attr.ib(type=RoomCreatedEventData, default=None)



@attr.s
class RoomDeletedEventData(object):
    room_id = attr.ib(type=str, default=None, metadata={'json': 'room_id'})
    room_name = attr.ib(type=str, default=None, metadata={'json': 'room_name'})


@attr.s
class RoomDeletedEvent(BaseEventV2):
    event = attr.ib(type=RoomDeletedEventData, default=None)



@attr.s
class RoomStatusChangedEventData(object):
    room_id = attr.ib(type=str, default=None, metadata={'json': 'room_id'})
    room_name = attr.ib(type=str, default=None, metadata={'json': 'room_name'})


@attr.s
class RoomStatusChangedEvent(BaseEventV2):
    event = attr.ib(type=RoomStatusChangedEventData, default=None)



@attr.s
class RoomUpdatedEventData(object):
    room_id = attr.ib(type=str, default=None, metadata={'json': 'room_id'})
    room_name = attr.ib(type=str, default=None, metadata={'json': 'room_name'})


@attr.s
class RoomUpdatedEvent(BaseEventV2):
    event = attr.ib(type=RoomUpdatedEventData, default=None)



@attr.s
class MeetingRoomStatusChangedEventData(object):
    room_name = attr.ib(type=str, default=None, metadata={'json': 'room_name'})
    room_id = attr.ib(type=str, default=None, metadata={'json': 'room_id'})


@attr.s
class MeetingRoomStatusChangedEvent(BaseEventV2):
    event = attr.ib(type=MeetingRoomStatusChangedEventData, default=None)



@attr.s
class MeetingRoomCreatedEventData(object):
    room_name = attr.ib(type=str, default=None, metadata={'json': 'room_name'})
    room_id = attr.ib(type=str, default=None, metadata={'json': 'room_id'})


@attr.s
class MeetingRoomCreatedEvent(BaseEventV2):
    event = attr.ib(type=MeetingRoomCreatedEventData, default=None)



@attr.s
class MeetingRoomDeletedEventData(object):
    room_name = attr.ib(type=str, default=None, metadata={'json': 'room_name'})
    room_id = attr.ib(type=str, default=None, metadata={'json': 'room_id'})


@attr.s
class MeetingRoomDeletedEvent(BaseEventV2):
    event = attr.ib(type=MeetingRoomDeletedEventData, default=None)



@attr.s
class MeetingRoomUpdatedEventData(object):
    room_name = attr.ib(type=str, default=None, metadata={'json': 'room_name'})
    room_id = attr.ib(type=str, default=None, metadata={'json': 'room_id'})


@attr.s
class MeetingRoomUpdatedEvent(BaseEventV2):
    event = attr.ib(type=MeetingRoomUpdatedEventData, default=None)



# -*- coding: UTF-8 -*-

from typing import Union, TypeVar, Generic, List

from ...model import OapiHeader
from ...context import Context
from ...consts import ERR_CODE_OK
import attr

T = TypeVar('T')


@attr.s
class ErrorDetail(object):
    key = attr.ib(type=str, default='')
    value = attr.ib(type=str, default='')


@attr.s
class PermissionViolation(object):
    key = attr.ib(type=str, default='')
    subject = attr.ib(type=str, default='')
    description = attr.ib(type=str, default='')


@attr.s
class FieldViolation(object):
    field = attr.ib(type=str, default='')
    value = attr.ib(type=str, default='')
    description = attr.ib(type=str, default='')


@attr.s
class ErrorHelp(object):
    url = attr.ib(type=str, default='')
    description = attr.ib(type=str, default='')


@attr.s
class ResponseError(object):
    details = attr.ib(type=List[ErrorDetail], default=None)
    permission_violations = attr.ib(type=List[PermissionViolation], default=None)
    field_violations = attr.ib(type=List[FieldViolation], default=None)
    helps = attr.ib(type=List[ErrorHelp], default=None)


class Response(Generic[T]):

    def __init__(self, ctx, code=ERR_CODE_OK, data=None, msg='', error=None):
        # type: (Context, int, T,  str, ResponseError) -> None
        self.ctx = ctx
        self.code = code
        self.data = data
        self.error = error
        self.msg = msg

    def get_header(self):  # type: () -> OapiHeader
        return self.ctx.get_header()

    def get_request_id(self):  # type: () -> Union[None, str]
        return self.ctx.get_request_id()

    def get_http_status_code(self):  # type: () -> Union[None, int]
        return self.ctx.get_http_status_code()



# -*- coding: UTF-8 -*-

from typing import Callable, Dict, Union

from ..model.card import Card
from ...config import Config
from ...context import Context


class Manager(object):
    """
    This handler manager maintains user-defined handler which can be called when webhook requests was triggered
    """

    app_id_to_handler = {}  # type: Dict[str, Callable[[Context, Config, Card], Union[None, Dict]]]

    @staticmethod
    def set_callback(conf, handler):
        # type: (Config, Callable[[Context, Config,  Card], Union[None, Dict]]) -> None
        Manager.app_id_to_handler[conf.app_settings.app_id] = handler

    @staticmethod
    def get_callback(conf):
        # type: (Config) -> Union[Callable[[Context, Config, Card], Union[None, Dict]], None]
        app_id = conf.app_settings.app_id
        return Manager.app_id_to_handler.get(app_id)



# -*- coding: UTF-8 -*-
# This file is originally modified from https://open.feishu.cn/document/ukTMukTMukTM/uUTNz4SN1MjL1UzM

import base64
import hashlib
import json

from Crypto.Cipher import AES


class AESCipher(object):
    def __init__(self, key):  # type: (bytes) -> None
        self.bs = AES.block_size
        self.key = hashlib.sha256(AESCipher.str_to_bytes(key)).digest()

    @staticmethod
    def str_to_bytes(data):  # type: (bytes) -> bytes
        u_type = type(b"".decode('utf8'))
        if isinstance(data, u_type):
            return data.encode('utf8')
        return data

    @staticmethod
    def _unpad(s):  # type: (bytes) -> bytes
        return s[:-ord(s[len(s) - 1:])]

    def decrypt(self, enc):  # type: (bytes) -> bytes
        iv = enc[:AES.block_size]
        cipher = AES.new(self.key, AES.MODE_CBC, iv)
        return self._unpad(cipher.decrypt(enc[AES.block_size:]))

    def decrypt_string(self, enc):  # type: (bytes) -> str
        enc = base64.b64decode(enc)
        return self.decrypt(enc).decode('utf8')


# encrypt = "P37w+VZImNgPEO1RBhJ6RtKl7n6zymIbEG1pReEzghk="
# cipher = AESCipher("test key")
# print("Decrypted string:\n{}".format(cipher.decrypt_string(encrypt)))

def decrypt(encrypt, key):  # type: (bytes, bytes) -> str
    json_obj = json.loads(encrypt)
    cipher = AESCipher(key)
    return cipher.decrypt_string(json_obj['encrypt'])



# -*- coding: UTF-8 -*-


from typing import Any, Callable, Dict, Union

from ...config import Config
from ...context import Context
from ..model.event import HTTPEvent


class Manager(object):
    """
    This handler manager maintains user-defined handler which can be called when webhook requests was triggered
    """

    event_callback_map = \
        {}  # type: Dict[str, Callable[[Context, Config, HTTPEvent], Any]]

    event_type_map = \
        {}  # type: Dict[str, Any]

    @staticmethod
    def set_event_callback(conf, event_type, handler, clazz):
        # type: (Config, str, Callable[[Context, Config, Any], Any], Any) -> None

        key = Manager.__key(conf.app_settings.app_id, event_type)
        Manager.event_callback_map[key] = handler
        Manager.event_type_map[key] = clazz

    @staticmethod
    def get_event_callback(conf, event_type):
        # type: (Config, str) -> Union[Callable[[Context, Config, HTTPEvent], Any], None]

        key = Manager.__key(conf.app_settings.app_id, event_type)
        return Manager.event_callback_map.get(key)

    @staticmethod
    def get_event_type(conf, event_type):
        # type: (Config, str) -> Union[Any, None]

        key = Manager.__key(conf.app_settings.app_id, event_type)
        return Manager.event_type_map.get(key)

    @staticmethod
    def __key(app_id, event_type):  # type: (str, str) -> str
        key = '%s-%s' % (app_id, event_type)
        return key



# -*- coding: UTF-8 -*-
# Code generated by lark suite oapi sdk gen

from typing import *

from ....api import Request, Response, set_timeout, set_tenant_key, set_user_access_token, set_path_params, \
    set_query_params, set_response_stream, set_is_response_stream, FormData, FormDataFile
from ....config import Config
from ....consts import ACCESS_TOKEN_TYPE_TENANT, ACCESS_TOKEN_TYPE_USER, ACCESS_TOKEN_TYPE_APP
from .model import *


class Service(object):
    def __init__(self, conf):
        # type: (Config) -> None
        self.conf = conf
        self.images = ImageService(self)
        



class ImageService(object):
    def __init__(self, service):
        # type: (Service) -> None
        self.service = service

    def get(self, tenant_key=None, response_stream=None, timeout=None):
        # type: (str, Union[None, IO], int) -> ImageGetReqCall

        request_opts = []   # type: List[Callable[[Any], Any]]

        if timeout is not None:
            request_opts += [set_timeout(timeout)]

        if tenant_key is not None:
            request_opts += [set_tenant_key(tenant_key)]

        if response_stream is not None:
            request_opts += [set_response_stream(response_stream)]

        return ImageGetReqCall(self, request_opts=request_opts)

    def put(self, tenant_key=None, timeout=None):
        # type: (str, int) -> ImagePutReqCall

        request_opts = []   # type: List[Callable[[Any], Any]]

        if timeout is not None:
            request_opts += [set_timeout(timeout)]

        if tenant_key is not None:
            request_opts += [set_tenant_key(tenant_key)]

        return ImagePutReqCall(self, request_opts=request_opts)



class ImageGetReqCall(object):
    def __init__(self, service, request_opts=None):
        # type: (ImageService, List[Any]) -> None

        self.service = service
        
        self.query_params = {}  # type: Dict[str, Any]

        if request_opts:
            self.request_opts = request_opts
        else:
            self.request_opts = []  # type: List[Any]

    def set_image_key(self, image_key):
        # type: (str) -> ImageGetReqCall
        self.query_params['image_key'] = image_key
        return self

    def do(self):
        # type: () -> Response[None]
        root_service = self.service.service

        conf = root_service.conf
        self.request_opts += [set_query_params(self.query_params)]
        self.request_opts += [set_is_response_stream()]
        req = Request('image/v4/get', 'GET', [ACCESS_TOKEN_TYPE_TENANT],
                      None, request_opts=self.request_opts)
        resp = req.do(conf)
        return resp


class ImagePutReqCall(object):
    def __init__(self, service, request_opts=None):
        # type: (ImageService, List[Any]) -> None

        self.service = service
        self.body = FormData()

        if request_opts:
            self.request_opts = request_opts
        else:
            self.request_opts = []  # type: List[Any]

    def set_image(self, image):
        # type: (IO[Any]) -> ImagePutReqCall
        self.body.add_file('image', FormDataFile(image))
        return self

    def set_image_type(self, image_type):
        # type: (str) -> ImagePutReqCall
        self.body.add_param('image_type', image_type)
        return self

    def do(self):
        # type: () -> Response[Image]
        root_service = self.service.service

        conf = root_service.conf
        req = Request('image/v4/put', 'POST', [ACCESS_TOKEN_TYPE_TENANT], self.body, output_class=Image , request_opts=self.request_opts)
        resp = req.do(conf)
        return resp

